﻿# Описание процесса запуска и работы модуля вычислений.

## Запуск

Модуль может быть запущен в двух режимах.

* В режиме нормальной работы
* В режиме импортирования данных

При запуске происходит определение идентификатора скважины с данными которой будет происходить работа:
- В режиме нормальной работы запрашивается идентификатор активной скважины и если таковой нет - работа модуля останавливается.
- В режиме импорта нужный идентификатор передается в параметрах запуска.

В дальнейшем вся работа с данными будет идти в контексте конкретной скважины.

Следующим шагом модуль запрашивает максимальную глубину скважины, список оперативных справочников и загружает все данные из этих справочников в память. 
Далее происходит загрузка и базовая проверка синтаксиса скриптов пред и постобработки, а также загружается список созданных переменных.
После формирования нужных массивов переменных запускается скрипт инициализации каждой переменной для которой он определен, а также все внешние переменные
получают текущий маркер времени последнего поступления данных. 

## Инициализация

Прототип скрипта инициализации: `function Recalculation(v,ref);` где:
- v   - объект содержащий все определенные в модуле переменные 
- ref - объект для работы со справочниками

При работе в нормальном режиме запускается таймер контроля таймаута.

Далее модуль переходит в режим готовности к работе и ожидает поступление данных.

## Перерасчет

Запуск скрипта перерасчета начинается с момента поступления данных. Источником данных могут быть:
* Точки интеграции
* Процесс импортирования

Данные поступают пакетом (не по одной переменной, а блоками). Каждое событие поступления данных имеет маркировку с названием отправителя (для точек интеграции это
содержимое поля настроек _"Маркер события"_, для импорта это значение "Import"), датой и текущей глубиной.

При получении такого пакета происходит первичный анализ поступивших данных в котором происходит конвертация полученных значений в нужные единицы измерения и проверка 
на то изменились ли данные (с учетом настроек _"терпимого расхождения"_).
Также обновляется маркер времени последнего поступления данных для задействованных переменных.
В случае если значение изменилось и превысило порог "терпимого расхождения" содержимое переменной обновляется, а в случае заполненного поля 
_"Логирование"_ данная переменная помечается как требующая запись в БД.

Если хоть одна переменная была изменена то происходит запуск скриптов вычислений по следующей схеме:

1. Создается и инициализируется объект заголовка
```
   let Hdr={
            EvName:  <Название отправителя>,
            Lag:     <Время запаздывания или 0 для импорта>,
            Date:    <ДатаВремя>,
            Depth:   <Глубина>,
            MaxDepth:<Максимально известная глубина>,
            Mode:    <Режим. 1-если Depth>MaxDepth иначе 0>
           }
```

2. Исполняется скрипт предобработки. Прототип скрипта:  `function PredProcessing(hdr,v,ref);` где:
	- hdr - созданный в первом шаге объект 
	- v   - объект содержащий все определенные в модуле переменные 
	- ref - объект для работы со справочниками

3. Исполняются последовательно сверху вниз все скрипты перерасчета для каждой переменной. В случае если значение которое вернул скрипт изменилось и превысило 
порог _"терпимого расхождения"_ содержимое переменной обновляется, а в случае заполненного поля _"Логирование"_ данная переменная помечается как требующая запись в БД.
Прототип скрипта: `function Recalculation(hdr,v,ref);` где:
	- hdr - созданный в первом шаге объект 
	- v   - объект содержащий все определенные в модуле переменные 
	- ref - объект для работы со справочниками

4. Исполняется скрипт постобработки. Прототип скрипта: `function PostProcessing(hdr,v,ref);` где: 
	- hdr - созданный в первом шаге объект 
	- v   - объект содержащий все определенные в модуле переменные 
	- ref - объект для работы со справочниками

5. Обновляем информацию об известной максимальной глубине

6. Записываем содержимое полей с пометкой _"требующая запись"_, в базу данных
 

При срабатывании таймера контроля таймаута алгоритм работы следующий:

Последовательно перебираются все внешние переменные для которых значение _"Таймаут в секундах"_ больше 0 и проверяется сколько прошло времени с момента их
обновления и в случае превышения указанного времени запускается скрипт обработки таймаута. 

Прототип скрипта: `function TimeOutProcessing(v,ref);` где:

 - v   - объект содержащий все определенные в модуле переменные 
 - ref - объект для работы со справочниками

Обновляется маркер последнего обновления данных 

В случае если значение которое вернул скрипт изменилось и превысило порог _"терпимого расхождения"_ содержимое переменной обновляется,
а в случае заполненного поля _"Логирование"_ данная переменная помечается как требующая запись в БД.

Если при обработке таймаута хотя бы одна переменная была изменена запускается общий скрипт перерасчета в этом случае **hdr.EvName** будет null.


Для работы со справочниками можно использовать следующие функции:

Функция | Описание
------------------------------------------ | -------------------------------------
ref.GetCount("<Мнемоническое имя спр.>"); | Возвращает число записей в справочнике
ref.GetMax("<Мнемоническое имя спр.>","<Мнемоническое имя поля>"); | Возвращает максимальное значение справочника указанного поля
ref.GetMin("<Мнемоническое имя спр.>","<Мнемоническое имя поля>"); | Возвращает минимальное значение справочника указанного поля
ref.GetLastValue("<Мнемоническое имя спр.>","<Мнемоническое имя поля>"); | Возвращает значение указанного поля из последней записи в справочнике
ref.GetFirstValue("<Мнемоническое имя спр.>","<Мнемоническое имя поля>"); | Возвращает значение указанного поля из первой записи в справочнике
ref.GetValue("<Мнемоническое имя спр.>","<Мнемоническое имя поля>",Index); | Возвращает значение указанного поля из записи с указанным индексом в справочнике. Отсчет индекса начинается с 0.
ref.GetSum("<Мнемоническое имя спр.>","<Мнемоническое имя поля>"); | Возвращает сумму всех значений указанного поля в справочнике
ref.GetAvg("<Мнемоническое имя спр.>","<Мнемоническое имя поля>"); | Возвращает среднее арифметическое значение указанного поля в справочнике

