CREATE OR REPLACE FUNCTION ApplayParamAcGrpFilter(i_acgrp DCDM_Users.AcGroup%TYPE,i_param varchar)
RETURNS varchar(20)[] IMMUTABLE LANGUAGE 'plpgsql'
AS $$
BEGIN
 return (select array( select name from unnest(string_to_array(i_param,',')) pn
 inner join dcdm_acgroupsfld a on a.name=pn.pn where a.acgroupid=i_acgrp));
END
$$;

GRANT EXECUTE ON FUNCTION  ApplayParamAcGrpFilter(DCDM_Users.AcGroup%TYPE,varchar) TO dcdmsrv;

---------------------------------------------------

CREATE INDEX dcdm_welllog_dpidx ON dcdm_welllog (wellid,depth);                               -- Индекс для ускорения выборки по максимальной глубине (без параметров)




drop FUNCTION GetFilteredCommentList;
CREATE OR REPLACE FUNCTION GetFilteredCommentList(
i_start bigint,	
i_wellid dcdm_commentsandfiles.wellid%TYPE,
i_orderby varchar(5),
i_isc bool,
i_isf bool,
i_tytes int[],
i_rt varchar(3),	
i_dfrom float8,
i_dto float8,	
i_tfrom timestamp,
i_tto timestamp	
)
RETURNS TABLE(
id dcdm_commentsandfiles.id%TYPE,
wid dcdm_commentsandfiles.wellid%TYPE,
uid	dcdm_commentsandfiles.userid%TYPE,
un  dcdm_users.name%TYPE,
crd float8,
chd float8,
t   dcdm_commentsandfiles.type%TYPE,
tn  dcdm_commentsandfilestype.name%TYPE,
isi dcdm_commentsandfiles.isindexed%TYPE,
sd  dcdm_commentsandfiles.snapdepth%TYPE,
svd dcdm_commentsandfiles.snapvdepth%TYPE,
st  float8,
msg dcdm_commentsandfiles.comment%TYPE,
fn  dcdm_commentsandfiles.orgfilename%TYPE,
fs  int
) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
DECLARE 
-- v_ordercolnum int;
 v_recperquery int:=100;
BEGIN
 -- Первый запрос вернет число записей
 IF i_start is NOT NULL then
  RETURN QUERY select i_start,count(*)::int,v_recperquery,null::varchar,null::float8,null::float8,
         null::smallint,null::varchar,null::bool,null::float8,null::float8,null::float8,null::varchar,
 		null::varchar,null::int
   from dcdm_commentsandfiles 
   where dcdm_commentsandfiles.wellid=i_wellid 
         AND dcdm_commentsandfiles.type=ANY(i_tytes) 
		 AND ((i_isc=true AND dcdm_commentsandfiles.comment IS NOT NULL) OR i_isc=false)
		 AND ((i_isf=true AND dcdm_commentsandfiles.filedata IS NOT NULL) OR i_isf=false)
		 AND ((i_rt='A')
 		     OR (i_rt='D' AND dcdm_commentsandfiles.snapdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='V' AND dcdm_commentsandfiles.snapvdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='T' AND dcdm_commentsandfiles.snaptime BETWEEN i_tfrom AND i_tto)
		    );
 end if;			
 -- Второй непосредственно результат			
 RETURN QUERY select dcdm_commentsandfiles.id,dcdm_commentsandfiles.wellid as wid,dcdm_commentsandfiles.userid as uid,dcdm_users.name as un,
                     date_part('epoch',dcdm_commentsandfiles.createtime) as crd,
					 date_part('epoch',dcdm_commentsandfiles.changetime) as chd,
					 dcdm_commentsandfiles.type as t,dcdm_commentsandfilestype.name as tn,
					 dcdm_commentsandfiles.isindexed as isi,
					 dcdm_commentsandfiles.snapdepth as sd,
					 dcdm_commentsandfiles.snapvdepth as svd,
					 date_part('epoch',dcdm_commentsandfiles.snaptime) as st,
					 dcdm_commentsandfiles.comment as msg,dcdm_commentsandfiles.orgfilename as fn,octet_length(dcdm_commentsandfiles.filedata)
  from dcdm_commentsandfiles 
       inner join dcdm_users on dcdm_users.id=dcdm_commentsandfiles.userid
	   inner join dcdm_commentsandfilestype on dcdm_commentsandfilestype.id=dcdm_commentsandfiles.type
  where dcdm_commentsandfiles.wellid=i_wellid 
        AND dcdm_commentsandfiles.type=ANY(i_tytes)
		AND ((i_isc=true AND dcdm_commentsandfiles.comment IS NOT NULL) OR i_isc=false)
		AND ((i_isf=true AND dcdm_commentsandfiles.filedata IS NOT NULL) OR i_isf=false)
		AND ((i_rt='A')
		     OR (i_rt='D' AND dcdm_commentsandfiles.snapdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='V' AND dcdm_commentsandfiles.snapvdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='T' AND dcdm_commentsandfiles.snaptime BETWEEN i_tfrom AND i_tto)
		    )
  order by 
   case when i_orderby='CrT' then dcdm_commentsandfiles.createtime end,
   case when i_orderby='ChT' then dcdm_commentsandfiles.changetime end,
   case when i_orderby='Dp' then dcdm_commentsandfiles.snapdepth end,
   case when i_orderby='VDp' then dcdm_commentsandfiles.snapvdepth end,
   case when i_orderby='EvT' then  dcdm_commentsandfiles.snaptime end
  limit case when i_start is not null then v_recperquery else null end offset case when i_start is not null then i_start else 0 end; 
END
$$;
GRANT EXECUTE ON FUNCTION  GetFilteredCommentList(bigint,dcdm_commentsandfiles.wellid%TYPE,
	   varchar(5),bool,bool,int[],varchar(3),float8,float8,timestamp,timestamp) TO dcdmsrv;



drop FUNCTION AddChangeCommentAndFile;
CREATE OR REPLACE FUNCTION AddChangeCommentAndFile(
i_id      dcdm_commentsandfiles.id%TYPE,
i_wellid  dcdm_commentsandfiles.wellid%TYPE,
i_userid  dcdm_commentsandfiles.userid%TYPE,
i_type    dcdm_commentsandfiles.type%TYPE,
i_sdepth  dcdm_commentsandfiles.snapdepth%TYPE,
i_svdepth dcdm_commentsandfiles.snapvdepth%TYPE,
i_stime   dcdm_commentsandfiles.snaptime%TYPE,
i_comment dcdm_commentsandfiles.comment%TYPE,
i_fname   dcdm_commentsandfiles.orgfilename%TYPE,
i_fdata   dcdm_commentsandfiles.filedata%TYPE
)
RETURNS TABLE(
id dcdm_commentsandfiles.id%TYPE,
wid dcdm_commentsandfiles.wellid%TYPE,
uid	dcdm_commentsandfiles.userid%TYPE,
un  dcdm_users.name%TYPE,
crd float8,
chd float8,
t   dcdm_commentsandfiles.type%TYPE,
tn  dcdm_commentsandfilestype.name%TYPE,
isi dcdm_commentsandfiles.isindexed%TYPE,
sd  dcdm_commentsandfiles.snapdepth%TYPE,
svd dcdm_commentsandfiles.snapvdepth%TYPE,
st  float8,
msg dcdm_commentsandfiles.comment%TYPE,
fn  dcdm_commentsandfiles.orgfilename%TYPE,
fs  int
) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_commentsandfiles(wellid,userid,type,snapdepth,snapvdepth,snaptime,comment,orgfilename,filedata) 
  VALUES (i_wellid,i_userid,i_type,i_sdepth,i_svdepth,i_stime,i_comment,i_fname,i_fdata);
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_commentsandfiles SET changetime=CURRENT_TIMESTAMP, type=i_type, isindexed=false,
                        snapdepth=i_sdepth,snapvdepth=i_svdepth,snaptime=i_stime,
						comment=i_comment--,orgfilename=i_fname,filedata=i_fdata
  WHERE dcdm_commentsandfiles.id=i_id AND dcdm_commentsandfiles.wellid=i_wellid AND dcdm_commentsandfiles.userid=i_userid;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY select dcdm_commentsandfiles.id,dcdm_commentsandfiles.wellid as wid,dcdm_commentsandfiles.userid as uid,dcdm_users.name as un,
                     date_part('epoch',dcdm_commentsandfiles.createtime) as crd,date_part('epoch',dcdm_commentsandfiles.changetime) as chd,dcdm_commentsandfiles.type as t,dcdm_commentsandfilestype.name as tn,
					 dcdm_commentsandfiles.isindexed as isi,dcdm_commentsandfiles.snapdepth as sd,dcdm_commentsandfiles.snapvdepth as svd,date_part('epoch',dcdm_commentsandfiles.snaptime) as st,
					 dcdm_commentsandfiles.comment as msg,dcdm_commentsandfiles.orgfilename as fn,octet_length(dcdm_commentsandfiles.filedata)
  from dcdm_commentsandfiles 
       inner join dcdm_users on dcdm_users.id=dcdm_commentsandfiles.userid
	   inner join dcdm_commentsandfilestype on dcdm_commentsandfilestype.id=dcdm_commentsandfiles.type
  where dcdm_commentsandfiles.id=i_id; 
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeCommentAndFile(dcdm_commentsandfiles.id%TYPE,dcdm_commentsandfiles.wellid%TYPE,dcdm_commentsandfiles.userid%TYPE,
							                        dcdm_commentsandfiles.type%TYPE,dcdm_commentsandfiles.snapdepth%TYPE,
												    dcdm_commentsandfiles.snapvdepth%TYPE,dcdm_commentsandfiles.snaptime%TYPE,dcdm_commentsandfiles.comment%TYPE,
												    dcdm_commentsandfiles.orgfilename%TYPE,dcdm_commentsandfiles.filedata%TYPE) TO dcdmsrv;




drop FUNCTION GetCommentsListByIds;
CREATE OR REPLACE FUNCTION GetCommentsListByIds(
i_start bigint,	
i_idslist varchar
)
RETURNS TABLE(
id dcdm_commentsandfiles.id%TYPE,
wid dcdm_commentsandfiles.wellid%TYPE,
uid	dcdm_commentsandfiles.userid%TYPE,
un  dcdm_users.name%TYPE,
crd float8,
chd float8,
t   dcdm_commentsandfiles.type%TYPE,
tn  dcdm_commentsandfilestype.name%TYPE,
isi dcdm_commentsandfiles.isindexed%TYPE,
sd  dcdm_commentsandfiles.snapdepth%TYPE,
svd dcdm_commentsandfiles.snapvdepth%TYPE,
st  float8,
msg dcdm_commentsandfiles.comment%TYPE,
fn  dcdm_commentsandfiles.orgfilename%TYPE,
fs  int,
pr	int
) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
DECLARE 
 v_recperquery int:=100;
BEGIN
 -- Первый запрос вернет число записей
 RETURN QUERY select i_start,count(*)::int,v_recperquery,null::varchar,null::float8,null::float8,
        null::smallint,null::varchar,null::bool,null::float8,null::float8,null::float8,null::varchar,
		null::varchar,null::int,null::int
  FROM unnest(i_idslist::varchar[]) list_id LEFT JOIN dcdm_commentsandfiles t on t.id=substring(list_id from '^(\d+)')::bigint;		
 -- Второй непосредственно результат			
 RETURN QUERY select dcdm_commentsandfiles.id,dcdm_commentsandfiles.wellid as wid,dcdm_commentsandfiles.userid as uid,dcdm_users.name as un,
                     date_part('epoch',dcdm_commentsandfiles.createtime) as crd,date_part('epoch',dcdm_commentsandfiles.changetime) as chd,dcdm_commentsandfiles.type as t,dcdm_commentsandfilestype.name as tn,
					 dcdm_commentsandfiles.isindexed as isi,dcdm_commentsandfiles.snapdepth as sd,dcdm_commentsandfiles.snapvdepth as svd,date_part('epoch',dcdm_commentsandfiles.snaptime) as st,
					 dcdm_commentsandfiles.comment as msg,dcdm_commentsandfiles.orgfilename as fn,octet_length(dcdm_commentsandfiles.filedata),
			         substring(list_id from ':(\d+)')::int as pr
  FROM unnest(i_idslist::varchar[]) list_id LEFT JOIN dcdm_commentsandfiles on dcdm_commentsandfiles.id=substring(list_id from '^(\d+)')::bigint		
       inner join dcdm_users on dcdm_users.id=dcdm_commentsandfiles.userid
	   inner join dcdm_commentsandfilestype on dcdm_commentsandfilestype.id=dcdm_commentsandfiles.type
 order by pr desc,dcdm_commentsandfiles.id desc	
  limit v_recperquery offset i_start; 
END
$$;
GRANT EXECUTE ON FUNCTION  GetCommentsListByIds(bigint,varchar) TO dcdmsrv;


-----------------------------------------------------------------------------------------
ALTER TABLE public.dcdm_vars  ADD COLUMN filter character varying;


drop FUNCTION AddChangeVarData;
SET plpgsql.extra_warnings TO 'shadowed_variables';
CREATE OR REPLACE FUNCTION AddChangeVarData(
 i_orgname dcdm_vars.sysname%TYPE,
 i_name dcdm_vars.sysname%TYPE,
 i_type dcdm_vars.type%TYPE,
 i_init dcdm_vars.initexpr%TYPE,
 i_calc dcdm_vars.calcexpr%TYPE,
 i_tlrdvg dcdm_vars.tlrdvg%TYPE,
 i_loggedpar dcdm_vars.loggedpar%TYPE,
 i_isext dcdm_vars.isexternal%TYPE,
 i_timeout dcdm_vars.timeout%TYPE,
 i_avgmethod dcdm_vars.avgmethod%TYPE,
 i_desc dcdm_vars.description%TYPE,
 i_lag dcdm_vars.lag%TYPE,
 i_group dcdm_vars.groupid%TYPE,
 i_filter dcdm_vars.filter%TYPE
)
RETURNS TABLE(
 sysname dcdm_vars.sysname%TYPE,
 type dcdm_vartype.name%TYPE,	
 groupid dcdm_vars.groupid%TYPE,
 groupname dcdm_vargroups.name%TYPE,
 description dcdm_vars.description%TYPE,
 loggedpar dcdm_vars.loggedpar%TYPE,
 isexternal dcdm_vars.isexternal%TYPE,
 avgmethod dcdm_vars.avgmethod%TYPE,
 sortpos dcdm_vars.sortpos%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE 
 sp dcdm_vars.sortpos%TYPE;
BEGIN
 IF i_orgname IS NULL then
  -- Добавление новой переменной 
  -- Получим максимальное значение позиции сортировки
  select COALESCE(MAX(dcdm_vars.sortpos),0)+1 from dcdm_vars into sp;
  -- Добавим переменную
  INSERT INTO dcdm_vars(sysname, groupid, type, initexpr, calcexpr, tlrdvg, loggedpar, isexternal, timeout, avgmethod, description, sortpos,lag,filter)
  VALUES (i_name,i_group,i_type,i_init,i_calc,i_tlrdvg,i_loggedpar,i_isext,i_timeout,i_avgmethod,i_desc,sp,i_lag,i_filter); 
 else
  -- редактирование
  UPDATE dcdm_vars SET sysname=i_name, groupid=i_group, type=i_type, initexpr=i_init, calcexpr=i_calc, 
   tlrdvg=i_tlrdvg, loggedpar=i_loggedpar, isexternal=i_isext, timeout=i_timeout, avgmethod=i_avgmethod, description=i_desc,
   lag=i_lag,filter=i_filter
  WHERE dcdm_vars.sysname=i_orgname;  
 end if;
 
 RETURN QUERY select dcdm_vars.sysname,dcdm_vartype.name as type,dcdm_vars.groupid,dcdm_vargroups.name as groupname ,dcdm_vars.description,dcdm_vars.loggedpar,dcdm_vars.isexternal,dcdm_vars.avgmethod,dcdm_vars.sortpos 
  from dcdm_vars 
  inner join dcdm_vartype on dcdm_vartype.id=dcdm_vars.type
  left outer join dcdm_vargroups on dcdm_vargroups.id=dcdm_vars.groupid
  where dcdm_vars.sysname=i_name;

 EXCEPTION 
  WHEN unique_violation then RAISE EXCEPTION 'Name must be unique' USING HINT='DCDM', DETAIL='VarNameError';
  WHEN check_violation then RAISE EXCEPTION 'Invalid character' USING HINT='DCDM', DETAIL='VarInvalidChar';
 
END
$$;

GRANT EXECUTE ON FUNCTION  AddChangeVarData(
 dcdm_vars.sysname%TYPE,dcdm_vars.sysname%TYPE,dcdm_vars.type%TYPE,dcdm_vars.initexpr%TYPE,dcdm_vars.calcexpr%TYPE,
 dcdm_vars.tlrdvg%TYPE,dcdm_vars.loggedpar%TYPE,dcdm_vars.isexternal%TYPE,dcdm_vars.timeout%TYPE,
 dcdm_vars.avgmethod%TYPE,dcdm_vars.description%TYPE,dcdm_vars.lag%TYPE,dcdm_vars.groupid%TYPE,dcdm_vars.filter%TYPE)
TO dcdmsrv;


drop function SortVar;
CREATE OR REPLACE FUNCTION SortVar(i_sysname dcdm_vars.sysname%TYPE,i_direction char(1))
RETURNS TABLE(
 sysname dcdm_vars.sysname%TYPE,
 type dcdm_vartype.name%TYPE,	
 groupid dcdm_vars.groupid%TYPE,
 groupname dcdm_vargroups.name%TYPE,
 description dcdm_vars.description%TYPE,
 loggedpar dcdm_vars.loggedpar%TYPE,
 isexternal dcdm_vars.isexternal%TYPE,
 avgmethod dcdm_vars.avgmethod%TYPE,
 sortpos dcdm_vars.sortpos%TYPE,
 filter dcdm_vars.filter%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 v_pos dcdm_vars.sortpos%TYPE;
 v_newpos dcdm_vars.sortpos%TYPE;
 v_psysname dcdm_vars.sysname%TYPE;
BEGIN
 select dcdm_vars.sortpos from dcdm_vars where dcdm_vars.sysname=i_sysname into v_pos;

 if i_direction='D' then
  select dcdm_vars.sortpos,dcdm_vars.sysname into v_newpos,v_psysname from dcdm_vars where dcdm_vars.sortpos>v_pos order by dcdm_vars.sortpos limit 1;
 elsif i_direction='U' then
  select dcdm_vars.sortpos,dcdm_vars.sysname into v_newpos,v_psysname from dcdm_vars where dcdm_vars.sortpos<v_pos order by dcdm_vars.sortpos desc limit 1;
 else
  v_newpos:=null;
  v_psysname:=null;
 end if; 
																					 
																					 
 if v_psysname IS NOT NULL AND v_newpos IS NOT NULL then
  UPDATE dcdm_vars SET sortpos=v_newpos WHERE dcdm_vars.sysname=i_sysname;
  UPDATE dcdm_vars SET sortpos=v_pos WHERE dcdm_vars.sysname=v_psysname;
 else 
  RAISE EXCEPTION 'Unknown sort direction' USING HINT='DCDM', DETAIL='VarSortError';
 end if; 

 RETURN QUERY select dcdm_vars.sysname,dcdm_vartype.name as type,dcdm_vars.groupid,dcdm_vargroups.name as groupname,
      dcdm_vars.description,dcdm_vars.loggedpar,dcdm_vars.isexternal,dcdm_vars.avgmethod,dcdm_vars.sortpos,dcdm_vars.filter 
  from dcdm_vars 
  inner join dcdm_vartype on dcdm_vartype.id=dcdm_vars.type
  left outer join dcdm_vargroups on dcdm_vargroups.id=dcdm_vars.groupid
  where dcdm_vars.sysname in(i_sysname,v_psysname);
END
$$;

GRANT EXECUTE ON FUNCTION  SortVar(dcdm_vars.sysname%TYPE,char) TO dcdmsrv;
