﻿CREATE EXTENSION IF NOT EXISTS "uuid-ossp";
CREATE EXTENSION IF NOT EXISTS tablefunc;
/*
Команды создания базы

CREATE USER dcdmsrv WITH LOGIN NOSUPERUSER NOCREATEDB NOCREATEROLE INHERIT NOREPLICATION CONNECTION LIMIT -1;
ALTER USER dcdmsrv PASSWORD '123';
CREATE TABLESPACE dcdmspace OWNER dcdmsrv LOCATION 'C:\dcdm_db';
ALTER TABLESPACE dcdmspace OWNER TO dcdmsrv;

CREATE DATABASE dcdm WITH OWNER = dcdmsrv TEMPLATE = template0 ENCODING = 'UTF8' LC_COLLATE = 'C' LC_CTYPE = 'C'  TABLESPACE = dcdmspace  CONNECTION LIMIT = -1;
*/
--#################################################################################################
Create table DCDM_Language(
 Lng  varchar(3)            NOT NULL PRIMARY KEY,                                                -- Код языка жалательно по ISO 639-3 но необязятельно
 Name varchar(50)           NOT NULL UNIQUE                                                      -- Название языка на этом самом языке
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Language TO dcdmsrv;
-- Исходные языки
INSERT INTO dcdm_language(lng, name) VALUES ('ru','Русский');
INSERT INTO dcdm_language(lng, name) VALUES ('en','English');
--#################################################################################################
Create table DCDM_VarType(
 Id  smallint            NOT NULL PRIMARY KEY,                     -- Идентификатор
 Name varchar(20)        NOT NULL UNIQUE                           -- Название
);

GRANT ALL PRIVILEGES ON TABLE DCDM_VarType TO dcdmsrv;
-- Стандартные типы
INSERT INTO DCDM_VarType(Id,Name) VALUES (1,'Int');
INSERT INTO DCDM_VarType(Id,Name) VALUES (2,'Float');
INSERT INTO DCDM_VarType(Id,Name) VALUES (3,'Text');
INSERT INTO DCDM_VarType(Id,Name) VALUES (4,'JSON');
--#################################################################################################
Create table DCDM_CommentsAndFilesType(
 Id  smallint            NOT NULL PRIMARY KEY,                     -- Идентификатор
 Name varchar            NOT NULL UNIQUE                           -- Название (мультиязычная строка)
);
GRANT ALL PRIVILEGES ON TABLE DCDM_CommentsAndFilesType TO dcdmsrv;

INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 1,':en Comment 1 :ru Комментарий 1 :es Comentario 1');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 2,':en Comment 2 :ru Комментарий 2 :es Comentario 2');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 3,':en Comment 3 :ru Комментарий 3 :es Comentario 3');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 4,':en Comment 4 :ru Комментарий 4 :es Comentario 4');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 5,':en Comment 5 :ru Комментарий 5 :es Comentario 5');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 6,':en Lithology :ru Литология :es Litología');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 7,':en Fluorescence :ru ЛБА :es Fluorescencia');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 8,':en Technological :ru Технологические :es Tecnológicos');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES ( 9,':en Timing :ru Хронометраж :es Tiempo');
INSERT INTO DCDM_CommentsAndFilesType(Id,Name) VALUES (10,':en Complications :ru Осложнения :es Complicaciones');
--#################################################################################################
Create table DCDM_AccessGroups(
  Id           serial       NOT NULL PRIMARY KEY,                                             -- Идентификатор
  Name         varchar      NOT NULL UNIQUE                                                  -- Название
);

GRANT ALL PRIVILEGES ON TABLE DCDM_AccessGroups TO dcdmsrv;
--#################################################################################################
Create table DCDM_AcGroupsFld(
  Id           serial       NOT NULL PRIMARY KEY,                                             -- Идентификатор
  AcGroupId    int          NOT NULL REFERENCES DCDM_AccessGroups(Id),                        -- Группа
  Name         varchar(20)  NOT NULL,                                                         -- Название поля журнада DCDM_WellLog
  UNIQUE(AcGroupId,Name)                                                              
);

GRANT ALL PRIVILEGES ON TABLE DCDM_AcGroupsFld TO dcdmsrv;
--#################################################################################################
Create table DCDM_Users(
  Id           serial       NOT NULL PRIMARY KEY,                                             -- Идентификатор
  Login        varchar(50)  NOT NULL UNIQUE,                                                  -- Логин
  Name         varchar(100) NULL,                                                             -- Имя
  PassHash     varchar(130) NOT NULL,                                                         -- Соленый хеш пароля
  Salt	       varchar(20) NOT NULL,                                                         -- Соль
  Permission   varchar(50)  NOT NULL,                                                         -- Права пользователя
  Config       json         NULL,                                                             -- Конфигурация интерфейса пользователя
  Props        json         NULL,                                                             -- Другие пользовательские настройки
  AcGroup      int          NULL REFERENCES DCDM_AccessGroups(Id),                            -- Группа доступа назначенная пользователю
  Lang         varchar(3)   NULL REFERENCES DCDM_Language(Lng),                               -- Язык интерфейса для пользователя
  IsDisabled   boolean      NOT NULL default false,                                           -- Флаг отключения аккаунта
  LatestUse    timestamp    NULL                                                              -- Время последнего входа
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Users TO dcdmsrv;
/*
ALTER TABLE DCDM_Users ADD COLUMN AcGroup int NULL;
ALTER TABLE DCDM_Users ADD CONSTRAINT fk_AcGroup FOREIGN KEY (AcGroup) REFERENCES DCDM_AccessGroups(Id);
*/

-- Встроенная запись администратора с паролем admin
INSERT INTO dcdm_users(login, name, PassHash, Salt, permission,lang)	VALUES ('Administrator','Built-in administrator account','2cbd97469403df5d34144359ddb6aaa3a3dbc06a676c1a974718e3f8f64e8e6e7627ee08f093f6f6fa5f3c96830bf927e9da27206c306eb0a9161fc11f9bde4e','a2d121cc6d76f44d','A','en');
--#################################################################################################
Create table DCDM_Messages(
  Id          bigserial     NOT NULL PRIMARY KEY,                                             -- Идентификатор
  UserId      int           NULL REFERENCES DCDM_Users(Id) ON DELETE SET NULL,                -- Идентификатор получателя
  IsReaded    boolean       NOT NULL DEFAULT false,                                           -- Признак прочитанного сообщения
  TypeCode    smallint      NOT NULL DEFAULT 0,                                               -- Код типа сообщения (для внутренних нужд 0-обчные сообщения 1-сообщения службы отчетов)
  Time        timestamp     NOT NULL DEFAULT CURRENT_TIMESTAMP,                               -- Дата и время создания сообщения
  SenderName  varchar(50)   NOT NULL,                                                         -- Отправитель
  Subj        varchar       NOT NULL,                                                         -- Тема сообщенич (HTML)
  Message     varchar       NULL                                                              -- Сообщение (HTML)
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Messages TO dcdmsrv;
--#################################################################################################
Create table DCDM_VarGroups(
  Id          serial        NOT NULL PRIMARY KEY,                                             -- Идентификатор
  Name        varchar       NULL,                                                             -- Название
  IsWidget    boolean       NOT NULL DEFAULT false,                                           -- Доступна в виджетах
  IsImport    boolean       NOT NULL DEFAULT false,                                           -- Доступна в импорте
  IsIPoint    boolean       NOT NULL DEFAULT false                                            -- Доступна в точках интеграции
);
GRANT ALL PRIVILEGES ON TABLE DCDM_VarGroups TO dcdmsrv;
--#################################################################################################
Create table DCDM_Vars(
  SysName     varchar(10)   NOT NULL PRIMARY KEY CHECK(SysName~* '^[A-Z].[A-Z0-9_]*$'),       -- Мнемоническое имя
  Type        smallint      NOT NULL REFERENCES DCDM_VarType(Id),                             -- Тип
  InitExpr    varchar       NULL,                                                             -- Строка инициализации (стартовое значение при запуске сервера)
  CalcExpr    varchar       NULL,                                                             -- Строка вычисления параметра или обработки таймаута (если это внешняя переменная)
  TlrDvg      float8        NULL,                                                             -- Терпимое расхождение (Переменная изменяется только если превышен этот порог. Актуально только для Int и Double типов)
  AvgMethod   int           NULL,                                                             -- Режим усреднения (0-отключен, >0-по указанной глубине, <0-по указанному времени в минутах)
  LoggedPar   varchar(20)   NULL,                                                             -- Название логируемого параметра или NULL если данные не логируются
  IsExternal  bool          NOT NULL,                                                         -- Флаг внешнего источника данных если он true то пересчет не используется
  Timeout     int           NULL,                                                             -- Время таймаута для внешней переменной
  Description varchar       NULL,                                                             -- Описание переменной (мультиязычное)
  SortPos     smallint      NOT NULL,                                                         -- Позиция сортировки
  Lag         varchar(10)   NULL,                                                             -- Запаздывание
  GroupId     int           NULL REFERENCES DCDM_VarGroups(Id),                               -- Идентификатор группы
  filter      varchar       NULL                                                              -- Фильтр
);

CREATE UNIQUE INDEX namecasecontrol ON DCDM_Vars (lower(SysName));
														
GRANT ALL PRIVILEGES ON TABLE DCDM_Vars TO dcdmsrv;
--#################################################################################################
Create table DCDM_Lags(
  SysName     varchar(10)   NOT NULL PRIMARY KEY CHECK(SysName~* '^[A-Z].[A-Z0-9_]*$'),       -- Мнемоническое имя
  InitVar     varchar(10)   NOT NULL REFERENCES DCDM_Vars(SysName),                           -- Переменная инициализации
  TruncVar    varchar(10)   NOT NULL REFERENCES DCDM_Vars(SysName)                            -- Переменная усечения
);

CREATE UNIQUE INDEX namelagcasecontrol ON DCDM_Lags (lower(SysName));
														
GRANT ALL PRIVILEGES ON TABLE DCDM_Lags TO dcdmsrv;

ALTER TABLE dcdm_vars
    ADD CONSTRAINT dcdm_vars_lag_fkey FOREIGN KEY (lag)
    REFERENCES dcdm_lags(sysname) MATCH SIMPLE
    ON UPDATE NO ACTION
    ON DELETE NO ACTION;
--#################################################################################################
Create table DCDM_Profiles(
  Id           serial       NOT NULL PRIMARY KEY,                                             -- Идентификатор
  Name         varchar(50)  NOT NULL UNIQUE,                                                  -- Название
  Description  varchar      NULL,                                                             -- Описание
  IsActive     boolean      NOT NULL DEFAULT false                                            -- Флаг активности профиля (активным должен быть только один)
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Profiles TO dcdmsrv;
--#################################################################################################
Create table DCDM_IPoints(
  Id           serial       NOT NULL PRIMARY KEY,                                             -- Идентификатор
  ProfileId    int          NOT NULL REFERENCES DCDM_Profiles(Id),                            -- Профиль
  PointType    varchar(20)  NOT NULL,                                                         -- Тип точки интерграции
  Name         varchar(50)  NOT NULL UNIQUE,                                                  -- Название
  Config       json         NOT NULL                                                          -- Конфигурация
);

GRANT ALL PRIVILEGES ON TABLE DCDM_IPoints TO dcdmsrv;
--#################################################################################################
Create table DCDM_ReferencesList(
  Id          serial        NOT NULL PRIMARY KEY,                                             -- Идентификатор
  SysName     varchar(10)   NOT NULL UNIQUE CHECK(SysName~* '^[A-Z].[A-Z0-9_]*$'),            -- Мнемоническое имя справочника. Например: Ref1
  Name        varchar       NOT NULL,                                                         -- Название справочника (мультиязычная строка)
  IsWellDep   boolean       NOT NULL,                                                         -- Скважинозависимый справочник (пустой для новой скважины)
  BrowseData  jsonb         NULL                                                              -- Информация о том откуда могут браться данные для данной таблице и как мапируются поля источника в данную таблицу
);

CREATE UNIQUE INDEX ON DCDM_ReferencesList (lower(SysName));
														
GRANT ALL PRIVILEGES ON TABLE DCDM_ReferencesList TO dcdmsrv;
--#################################################################################################
-- Описание колонки отдельно взятого справочника
Create table DCDM_ReferenceCol(
  Id          serial        NOT NULL PRIMARY KEY,                                             -- Идентификатор
  RefListId   int           NOT NULL REFERENCES DCDM_ReferencesList(Id),                      -- Идентификатор справочника в реестре
  Type          smallint      NOT NULL REFERENCES DCDM_VarType(Id),                           -- Тип
  SysName     varchar(10)   NOT NULL CHECK(SysName~* '^[A-Z].[A-Z0-9_]*$'),                   -- Мнемоническое имя поля. Например: Fld1
  ColName     varchar       NOT NULL,                                                         -- Название параметра (мультиязычная строка)
  InitExpr    varchar       NULL,                                                             -- Строка инициализации. Актуально только для скважинозависимых справочников
  UNIQUE(RefListId,SysName)                                                              
);

CREATE UNIQUE INDEX ON DCDM_ReferenceCol (lower(SysName));
GRANT ALL PRIVILEGES ON TABLE DCDM_ReferenceCol TO dcdmsrv;
--#################################################################################################
Create table DCDM_Well(
  Id          serial        NOT NULL PRIMARY KEY,                                             -- Идентификатор
  Name        varchar       NOT NULL,                                                         -- Название скважины (мультиязычное)
  Latitude    real          NULL,                                                             -- Широта
  Longitude   real          NULL,                                                             -- Долгота
  CrDate      timestamp     NOT NULL default current_timestamp,                               -- Дата создания
  IsActive    boolean       NOT NULL default false 	                                        -- признак активной скважины
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Well TO dcdmsrv;
--#################################################################################################
-- Данные српавиочников
Create table DCDM_ReferenceData(
  Id          bigserial     NOT NULL PRIMARY KEY,                                             -- Идентификатор
  WellId      int           NULL REFERENCES DCDM_Well(Id) ON DELETE CASCADE,                 -- Идентификатор скважины. (Актуально для скважинозависибых справочников)
  RefListId   int           NOT NULL REFERENCES DCDM_ReferencesList(Id) ON DELETE CASCADE,   -- Идентификатор справочника в реестре
  SubId       int           NOT NULL,                                                         -- Субидентификатор сортировки спрвочников
  GrpName     varchar(10)   NULL CHECK(GrpName~* '^[A-Z].[A-Z0-9_]*$'),                       -- Префикс для образования массива данных. Все поля с одним префиксом входят в один массив
  Val         json          NOT NULL                                                          -- json значение
);

CREATE UNIQUE INDEX ON DCDM_ReferenceData(WellId,RefListId,SubId) WHERE WellId IS NOT NULL;
CREATE UNIQUE INDEX ON DCDM_ReferenceData(RefListId,SubId) WHERE WellId IS NULL;

GRANT ALL PRIVILEGES ON TABLE DCDM_ReferenceData TO dcdmsrv;
--#################################################################################################
Create table DCDM_WellLogFieldDesc(
 ParamName  varchar(20)  NOT NULL PRIMARY KEY,                                                -- Название поля
 Descr      varchar      NULL                                                                 -- Мультиязычное описание
);
GRANT ALL PRIVILEGES ON TABLE DCDM_WellLogFieldDesc TO dcdmsrv;

INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AAFV',':en Acc. Actual Displacement :es Desplazamiento Actual Acumulado :ru Накопленное фактическое наполнение :cn 累积实际加入 :pt Acc. Deslocamento Atual');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AFV',':en Actual Displacement :es Desplazamiento Actual :ru Фактическое наполнение :cn 实际加入 :pt Deslocamento Atual');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AH2S1',':en Ambient H2S 1 :es H2S 1 Ambiente :ru Окружающий сероводород 1 :cn 大气中硫化񬈡 :pt H2S 1 Ambiente');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AH2S2',':en Ambient H2S 2 :es H2S 2 Ambiente :ru Окружающий сероводород 2 :cn 大气中硫化񬈢 :pt H2S 2 Ambiente');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AH2S3',':en Ambient H2S 3 :es H2S 3 Ambiente :ru Окружающий сероводород 3 :cn 大气中硫化񬈣 :pt H2S 3 Ambiente');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AH2S4',':en Ambient H2S 4 :es H2S 4 Ambiente :ru Окружающий сероводород 4 :cn 大气中硫化񬈤 :pt H2S 4 Ambiente');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AMD',':en Actual Mud Density');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AP1',':en Air Pressure GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AP2',':en Air Pressure GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('AROP',':en ROP Rolling :es ROP :ru Скорость проходки вращением :cn 机械钻速滚动 :pt ROP Rotação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BBT1',':en Enclosure Temp GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BBT2',':en Enclosure Temp GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCB1',':en Benzene Out :es Benzeno de Salida :pt Benzeno de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCB2',':en Benzene In :es Benzeno de Entrada :pt Benzeno de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCBD',':en Delta Benzene :es Delta Benzeno :cn Benzene Delta :pt Delta Benzeno');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCON',':en Bit Condition :es Condición de Mecha :ru Состояние бура :cn 钻头条件 :pt Condição da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCR1',':en Benzene Out(Raw) :es Benzeno de Salida (Raw) :pt Benzeno de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCR2',':en Benzene In(Raw) :es Benzeno de Entrada (Raw) :pt Benzeno de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCRD',':en Delta Benzene(Raw) :es Delta Benzeno (Raw) :pt Delta Benzeno (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCW1',':en QC Benzene Out :es QC Benzeno de Salida :pt QC Benzeno de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCW2',':en QC Benzene In :es QC Benzeno de Entrada :pt QC Benzeno de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BCWD',':en QC Delta Benzene :es QC Delta Benzeno :pt QC Delta Benzeno');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BDEN',':en Bulk Density :es Densidad Volumétrica :ru Объемная плотность :cn 体积密度 :pt Volumen de Densidade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BDIR',':en Block Direction :es Dirección del Bloque :ru Направление блока :cn 游车方向 :pt Direção do Bloco');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BDTI',':en Bit Drilled Time :es Tiempo de Perforación de la Mecha :ru Время бурения :cn 钻头使用时间 :pt Tempo de Perfuração da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BHALEN',':en BHA Length :es Longitud del BHA :ru Длина скважинного узла :cn 钻具组合长度 :pt Comprimento do BHA');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BHP',':en Hydraulic HP @ Bit :es Hidráulica HP @ Mecha :ru Гидравлический бур высокого давления :cn 钻头水马力 :pt Hidráulica HP @ Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BHPA',':en Bit Hydraulic Power/Area :es Área/Poder de la Mecha :ru Мощность/площадь гидравлического бура :cn 钻头水功率 :pt Àrea / Broca Poder de Hidráulica');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLAF',':en BL Air Flow');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLCS',':en BL Calibration Slope');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLDC',':en BL Detector Current');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLDT',':en BL Detector Temperature');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLFF',':en BL Fuel Flow');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLSF',':en BL Sample Flow');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLTG',':en BL Total Gas Raw');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLTGM',':en BL Total Gas Measured');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BLZO',':en BL Zero Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BMF_LAGGED',':en Lagged Channels :es Canales de Retraso :pt Canais de Atrasos (Lagged)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BMF_RECALCULATED',':en Recalculated Channels :es Canales Recalculados :pt Canais Recalculados');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BMOS',':en Block Movement Out-Of-Slips :es Movimiento del Bloque (Fuera de Cu༚s) :ru Отсутствуют смещения дивжения блока :cn 无卡瓦时游车运动 :pt Movimento do Bloco (Out-Of-Slips)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BMT',':en Block Movement :es Movimiento del Bloque :ru Движение блока :cn 游车运动 :pt Movimento do Bloco');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BOPT',':en Bit Off Bottom Time :es Tiempo de Mecha Fuera de Fondo :ru Время низа бура :cn 钻头非钻进时间 :pt Tempo da Broca Fora de Fundo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BP1',':en Enclosure Pressure GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BP2',':en Enclosure Pressure GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BPFI',':en Booster Pump Flow :es Flujo de Bomba Booster :ru Поток бустерного насоса :cn 泵启动流量 :pt Fluxo da Bomba do (Booster)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BPOS',':en Block Position :es Posición del Bloque :ru Позиция блока :cn 游车位置 :pt Posição do Bloco');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BPVM',':en Booster Pump Volume Moved :es Volumen Movido por la Bomba Booster :pt Volumen Movido da Bomba do (Booster)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BRAT',':en Bal R MLS');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BRAT2',':en Bal R GCT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BRI',':en Block Rate Instantaneous :es Rata Instantánea de Movimiento del Bloque :ru Мнгновенная частота блоков :cn 瞬时游车速度 :pt Taxa Instantânea do Bloco');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BRMC',':en Total Bit Distance :es Metros Totales de la Mecha :pt Metros Total da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BRTI',':en Bit Reamed Time :es Tiempo de Repaso de la Mecha :ru Время развёртывания бура :cn 钻头划眼时间 :pt Tempo Repassado da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BRVC',':en Total Bit Revolutions :es Revoluciones Totales de la Mecha :ru Общее число оборотов бура :cn 钻头总转数 :pt Revoluçཞs Total da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BTMP',':en Block Temperature :es Temperatura del Bloque :ru Температура блока :cn 游车温度 :pt Temperatura do Bloco');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BTPOSBM',':en Bit Position due to Block Movement :es Posición de la Mecha debida al Movimiento del Bloque :ru Позиция бура к движению блока :cn 因游车运动的钻头位置 :pt Posição Adequada da Broca para Movimento do Bloco');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BTTI',':en Bit Total Time :es Tiempo Total de la Mecha :ru Общее время бура :cn 总钻头使用时间 :pt Tempo Total da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('BWEAR',':en Bit Wear :es Desgaste de la Mecha :ru Износ бура :cn 钻头磨损 :pt Desgaste da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1C2',':en C1/C2 (Chromat)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1C3',':en C1/C3 (Chromat)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1C4',':en C1/C4 (Chromat)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1C5',':en C1/C5 (Chromat)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CB1',':en Methane Out :es Metano de Salida :pt Metano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CB2',':en Methane In :es Metano de Entrada :pt Metano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CBD',':en Delta Methane :es Delta Metano :pt Delta Metano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CFG',':en C1 Corrected For Free Gas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CR1',':en Methane Out(Raw) :es Metano de Salida (Raw) :pt Metano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CR2',':en Methane In(Raw) :es Metano de Entrada (Raw) :pt Metano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CRD',':en Delta Methane(Raw) :es Delta Metano (Raw) :pt Delta Metano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CW1',':en QC Methane Out :es QC Metano de Salida :pt QC Metano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CW2',':en QC Methane In :es QC Metano de Entrada :pt QC Metano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1CWD',':en QC Delta Methane :es QC Delta Metano :pt QC Delta Metano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1D',':en C1 Density');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1FC',':en C1 Flow Corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1PC',':en C1 Pressure Compensated');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1SF',':en C1 Solubility Factor');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C1THC',':en C1/THC (Chromat)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2C3',':en C2/C3 (Chromat)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CB1',':en Ethane Out :es Etano de Salida :pt Etano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CB2',':en Ethane In :es Etano de Entrada :pt Etano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CBD',':en Ethane Delta :es Delta Ethano :ru Delta Ethane :pt Delta Etano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CR1',':en Ethane Out(Raw) :es Etano de Salida (Raw) :pt Etano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CR2',':en Ethane In(Raw) :es Etano de Entrada (Raw) :pt Etano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CRD',':en Delta Ethane(Raw) :es Delta Etano (Raw) :pt Delta Etano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CW1',':en QC Ethane Out :es QC Etano de Salida :pt QC Etano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CW2',':en QC Ethane In :es QC Etano de Entrada :pt QC Etano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2CWD',':en QC Delta Ethane :es QC Delta Etano :pt QC Delta Etano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2FC',':en C2 Flow Corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C2PC',':en C2 Pressure Compensated');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CB1',':en Propane Out :es Propano de Salida :pt Propano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CB2',':en Propane In :es Propano de Entrada :pt Propano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CBD',':en Delta Propane :es Delta Propano :pt Delta Propano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CR1',':en Propane Out(Raw) :es Propano de Salida (Raw) :pt Propano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CR2',':en Propane In(Raw) :es Propano de Entrada (Raw) :pt Propano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CRD',':en Delta Propane(Raw) :es Delta Propano (Raw) :pt Delta Propano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CW1',':en QC Propane Out :es QC Propano de Salida :pt QC Propano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CW2',':en QC Propane In :es QC Propano de Entrada :pt QC Propano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3CWD',':en QC Delta Propane :es QC Delta Propano :pt QC Delta Propano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3FC',':en C3 Flow Corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C3PC',':en C3 Pressure Compensated');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CB1',':en N-Hexane Out :es N-Hexano de Salida :pt N-Hexano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CB2',':en N-Hexane In :es N-Hexano de Entrada :pt N-Hexano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CBD',':en Delta N-Hexane :es Delta N-Hexano :pt Delta N-Hexano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CR1',':en N-Hexane Out(Raw) :es N-Hexano de Salida (Raw) :pt N-Hexano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CR2',':en N-Hexane In(Raw) :es N-Hexano de Entrada (Raw) :pt N-Hexano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CRD',':en Delta N-Hexane(Raw) :es Delta N-Hexano (Raw) :pt Delta N-Hexano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CW1',':en QC N-Hexane Out :es QC N-Hexano de Salida :pt QC N-Hexano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CW2',':en QC N-Hexane In :es QC N-Hexano de Entrada :pt QC N-Hexano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C6CWD',':en QC Delta N-Hexane :es QC Delta N-Hexano :pt QC Delta N-Hexano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CB1',':en N-Heptane Out :es N-Heptano de Salida :pt N-Heptano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CB2',':en N-Heptane In :es N-Heptano de Entrada :pt N-Heptano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CBD',':en Delta N-Heptane :es Delta N-Heptano :pt Delta N-Heptano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CR1',':en N-Heptane Out(Raw) :es N-Heptano de Salida (Raw) :pt N-Heptano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CR2',':en N-Heptane In(Raw) :es N-Heptano de Entrada (Raw) :pt N-Heptano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CRD',':en Delta N-Heptane(Raw) :es Delta N-Heptano (Raw) :pt Delta N-Heptano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CW1',':en QC N-Heptane Out :es QC N-Heptano de Salida :pt QC N-Heptano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CW2',':en QC N-Heptane In :es QC N-Heptano de Entrada :pt QC N-Heptano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7CWD',':en QC Delta N-Heptane :es QC Delta N-Heptano :pt QC N-Heptano Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCB1',':en Methylcyclohexane Out :es Methylcyclohexano de Salida :pt Methylcyclohexano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCB2',':en Methylcyclohexane In :es Methylcyclohexano de Entrada :pt Methylcyclohexano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCBD',':en Delta Methylcyclohexane :es Delta Methylcyclohexano :pt Delta Methylcyclohexano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCR1',':en Methylcyclohexane Out(Raw) :es Methylcyclohexano de Salida (Raw) :pt Methylcyclohexano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCR2',':en Methylcyclohexane In(Raw) :es Methylcyclohexano de Entrada (Raw) :pt Methylcyclohexano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCRD',':en Delta Methylcyclohexane(Raw) :es Delta Methylcyclohexano (Raw) :pt Delta Methylcyclohexano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCW1',':en QC Methylcyclohexane Out :es QC Methylcyclohexano de Salida :pt QC Methylcyclohexano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCW2',':en QC Methylcyclohexane In :es QC Methylcyclohexano de Entrada :pt QC Methylcyclohexano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C7HCWD',':en QC Delta Methylcyclohexane :es QC Delta Methylcyclohexano :pt QC Delta Methylcyclohexano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CB1',':en N-Octane Out :es N-Octano de Salida :pt N-Octano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CB2',':en N-Octane In :es N-Octano de Entrada :pt N-Octano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CBD',':en Delta N-Octane :es Delta N-Octano :pt Delta N-Octano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CR1',':en N-Octane Out(Raw) :es N-Octano de Salida (Raw) :pt N-Octano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CR2',':en N-Octane In(Raw) :es N-Octano de Entrada (Raw) :pt N-Octano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CRD',':en Delta N-Octane(Raw) :es Delta N-Octano (Raw) :pt Delta N-Octano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CW1',':en QC N-Octane Out :es QC N-Octano de Salida :pt QC N-Octano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CW2',':en QC N-Octane In :es QC N-Octano de Entrada :pt QC N-Octano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('C8CWD',':en QC Delta N-Octane :es QC Delta N-Octano :pt QC Delta N-Octano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CALP1',':en CalGas Pressure GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CALP2',':en CalGas Pressure GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:AN',':en Anadrill Record 63 :es Registro 63 Anadrill :ru Запись Anadrill 63 :cn Anadrill 记� :pt Registro 63 Anadrill');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:BI',':en Bit Parameters :es Parámetros de Mecha :ru Параметры бура :cn 钻头参数 :pt Parâmetros da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:CM',':en Comments :es Comentarios :ru Комментарии :cn 备注 :pt Comentários');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:DB',':en Debug Parameters :es Parámetros de Debug :ru Отладочные параметры :cn 调试参数 :pt Debug Parâmetros');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:DE',':en Depth Parameters :es Parámetros de Profundidad :ru Параметры глубины :cn 井深参数 :pt Parâmetros de Profundidade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:DP',':en Drilling Parameters :es Parámetros de Perforación :ru Параметры бурения :cn 钻井参数 :pt Parâmetros de Perfuração');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:EHYD',':en eHydraulics Parameters :es Parámetros de eHydraulics :pt Parâmetros eHidráulica');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:GA',':en GCT Gas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:GCTH',':en GCT Hardware');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:MPD',':en GCT MPD');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:GCTR',':en GCT Ratios');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:LI',':en Lithology :es Litolog໚ :ru Литология :cn 岩性 :pt Litologia');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:LP',':en Lag Parameters');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:MI',':en Miscellaneous Parameters :es Parámetros Diversos :ru Смешанные параметры :cn 其它参数 :pt Parâmetros Diversos');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:MLSG',':en MLS Gas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:MU',':en Mud Parameters :es Parámetros de Lodo :ru Параметры бурильного раствора :cn 泥浆参数 :pt Parâmetros de Lama');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:MWD',':en MWD Parameters :es Parámetros de MWD :ru Параметры скважинных исследований :cn MWD参数 :pt Parâmetros de MWD');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:NORTHLAND',':en Northland Energy :ru Северная энергия :pt Energia Northland');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:PI',':en Pit Parameters :es Parámetros de Tanques :ru Параметры шахты :cn 泥浆池参数 :pt Parâmetros de Volumens');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:PIPE',':en Pipe Parameters :es Parámetros de Tuber໚ :ru Параметры трубы :cn 管线参数 :pt Parâmetros de Tubos (Pipe)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:PR',':en Pressure Parameters :es Parámetros de Presión :ru Параметры давления :cn 压力参数 :pt Parâmetros de Pressão');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:PU',':en Pump Parameters :es Parámetros de Bombas :ru Параметры насоса :cn 泵参数 :pt Parâmetros de Bomba');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:TP',':en Trip Parameters :es Parámetros de Viaje :pt Parâmetros de Viagem');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:WL',':en Wireline Parameters :es Parámetros de Wireline :ru Линейные параметры :cn 电缆参数 :pt Parâmetros Wireline');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CAT:WWSRA',':en Source Rock Analyzer');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CBG1',':en Combustible 1 :ru Горючее 1 :cn 可燃񠉱 :pt Combustível 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CBG2',':en Combustible 2 :ru Горючее 2 :cn 可燃񠉲 :pt Combustível 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CBG3',':en Combustible 3 :ru Горючее 3 :cn 可燃񠉳 :pt Combustível 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CBG4',':en Combustible 4 :ru Горючее 4 :cn 可燃񠉴 :pt Combustível 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CBWEAR',':en Cumulative Bit Wear :es Desgaste Acumulado de la Mecha :ru Совокупный износ бура :cn 累积的钻头磨损 :pt Cumulativo Visto da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CCGS',':en CC Gas Sensor :es Sensor de Gas CC :ru Сенсор газовой критической трещины :cn 催化燃烧气体分析仪 :pt CC Sensor de Gàs');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CCQC',':en CC QC Ratio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CDO',':en Calcimetry Dolomite :es Calcimetr໚ de Dolomitas :ru Доломитовая кальциметрия :cn 含碳酸盐的白云岩 :pt Calcimetria de Dolomite');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CHKP',':en Casing Pressure :es Presión de Revestidor :ru Давление в кольцевом пространстве :cn 套压 :pt Pressão de Casing');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CHOKEP',':en Choke Pressure :es Presión de Choque :pt Pressão de Choke');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CLD',':en Chromat Lag Depth');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CLI',':en Calcimetry Limestone :es Calcimetr໚ de Carbonatos :ru Известняковая кальциметрия :cn 含碳酸盐的石灰岩 :pt Calcimetria Limestone');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CLT',':en Chromat Lag Time');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO',':en CO :ru Моноокись углерода :cn 一氧化碳');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2',':en CO2 :ru Двуокись углерода :cn 二氧化碳');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CB1',':en Carbon Dioxide Out :es Dióxido de Carbono de Salida');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CB2',':en Carbon Dioxide In :es Dióxido de Carbono de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CBD',':en Delta Carbon Dioxide :es Delta Dióxido de Carbono');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CR1',':en Carbon Dioxide Out(Raw) :es Dióxido de Carbono de Salida (Raw) :pt Gás Carbônico de Saida (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CR2',':en Carbon Dioxide In(Raw) :es Dióxido de Carbono de Entrada (Raw) :pt Gás Carbônico de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CRD',':en Delta Carbon Dioxide(Raw) :es Delta Dióxido de Carbono (Raw) :pt Gás Carbônico Delta (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CW1',':en QC Carbon Dioxide Out :es QC Dióxido de Carbono de Salida :pt Gás Carbônico de Saida');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CW2',':en QC Carbon Dioxide In :es QC Dióxido de Carbono de Entrada :pt Gás Carbônico de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2CWD',':en QC Delta Carbon Dioxide :es QC Delta Dióxido de Carbono :pt Gás Carbônico Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2_1',':en CO2 Out :es CO2 de Salida :ru Двуокись углерода исходящая :cn 排出二氧化碳 :pt CO2 de Saida');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO2_2',':en CO2 In :es CO2 de Entrada :ru Двуокись углерода входящая :cn 输入二氧化碳 :pt CO2 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM1',':en Comment1 :es Comentario1 :ru Комментарий 1 :cn 备񬺁 :pt Comentário 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM2',':en Comment2 :es Comentario 2 :ru Комментарий 2 :cn 备񬺂 :pt Comentário 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM3',':en Comment3 :es Comentario 3 :ru Комментарий 3 :cn 备񬺃 :pt Commentário 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM4',':en Comment4 :es Comentario 4 :ru Комментарий 4 :cn 备񬺄 :pt Commentário 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM5',':en Comment5 :es Comentario 5 :ru Комментарий 5 :cn 备񬺅 :pt Commentário 5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM6',':en Comment6 :es Comentario 6 :ru Комментарий 6 :cn 备񬺆 :pt Commentário 6');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM7',':en Comment7 :es Comentario 7 :ru Комментарий 7 :cn 备񬺇 :pt Commentário 7');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM8',':en Comment8 :es Comentario 8 :ru Комментарий 8 :cn 备񬺈 :pt Commentário 8');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM9',':en SRA Comment :es Comentario 9 :ru Комментарий 9 :cn 备񬺉 :pt Commentário 9');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COM10',':en Comment10 :es Comentario 10 :ru Комментарий 10 :cn 备� :pt Commentário 10');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COMD',':en Depth Comment :es Comentario de Profundidad :ru Комментарий глубины :cn 井深备注 :pt Comentário de Profundidade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COMI',':en Interpreted Lithology Comment :es Comentario de Litolog໚ Interpretada :ru Интерпретированный комментарий литологии :cn 岩性解释备注 :pt Comentàrios de Litologia Interpretada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COMM',':en Memos :ru Заметки :cn 备忘录');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COMP',':en Percentage Lithology Comment :es Comentario de Litolog໚ Porcentual :ru Комментарий процентной литологии :cn 岩性百分比注释 :pt Comentários de Litologia Porcentual');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('COMT',':en Time Comment :es Comentario de Tiempo :ru Временной комментарий :cn 时间备注 :pt Comentário Tempo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO_1',':en CO Out :es CO de Salida :ru Моноокись углерода исходящая :cn 排出一氧化碳 :pt CO Saida');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CO_2',':en CO In :es CO de Entrada :ru Моноокись углерода входящая :cn 输入一氧化碳 :pt CO Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CP1',':en Carrier Pressure GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CP2',':en Carrier Pressure GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CPDC',':en Cost/Distance :es Costo/Distancia :ru Цена/Расстояние :cn 花费/距离 :pt Custo/Distância');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CPOS',':en Comp Position :es Posición del Compensador :ru Расчётная позиция :cn 位置补偿 :pt Comp. Posição');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CRAT',':en Char R MLS');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CRAT2',':en Char R GCT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CT1',':en Chromat Temp GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CT2',':en Chromat Temp GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CTTL',':en Chromat Time to Lag');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CWTD',':en Tracer Delta Mud Temp :es Tracer - Temperatura de Lodo Delta :pt Tracer Temp Lama Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CWTI',':en Tracer In Mud Temp :es Tracer - Temperatura de Lodo de Entra :pt Tracer Temp de Entrada da Lama');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('CWTO',':en Tracer Out Mud Temp :es Tracer - Temperatura de Lodo de Salida :pt Tracer Temp de Saida da Lama');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DAYNO',':en Day Number :es Número de D໚ :ru Номер дня :cn 天数 :pt Número de Dia');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DBSD',':en Bit Start Depth :es Inicio de Profundidad de la Mecha :ru Стартовая глубина бура :cn 钻头起始位置 :pt Inicio de Profundidade da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DBST',':en Bit Start Time :es Tiempo de Inicio de la Mecha :ru Стартовое время бура :cn 钻头起始时间 :pt Inicio do Tempo da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DBTM',':en Bit Depth :es Profundidad de la Mecha :ru Глубина бура :cn 钻头井深 :pt Profundidade da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DBTMKB',':en Bit Depth TRKB :es Profundidad de la Mecha TRKB :ru Глубина бура истинный относительный ротор :cn 钻头井深(实际转盘补芯) :pt Profundidade da Broca TRKB');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DCV1',':en System DC Voltage GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DCV2',':en System DC Voltage GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DGM',':en Depth G.Ray MWD :es Profundidad de MWD Gamma Ray :ru Глубина гамма-лучей измеренная во время бурения :cn 随钻测量伽马井深 :pt Profundidade G.Ray MWD');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DGM2',':en Depth G. Ray 2 MWD');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DGM2O',':en Gamma 2 Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DGMO',':en Gamma Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DIPA',':en Dip Angle :es Angulo DIP :cn 倾角 :pt Dip Ângulo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DMEA',':en Hole Depth :es Profundidad de Hoyo :ru Глубина скважины :cn 井深 :pt Profundidade do Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DMEAKB',':en Hole Depth TRKB :es Profundidad de Hoyo TRKB :ru Глубина скважины истинный относительный ротор :cn 井深(实际转盘补芯) :pt Profundidade do Poço TRKB');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DP1M',':en Porosity Depth 1 :es Profundidad 1 de Porosidad :pt Profundidade 1 Porosidade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DP1MO',':en Porosity 1 Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DP2M',':en Porosity Depth 2 :es Profundidad 2 de Porosidad :pt Profundidade 2 Porosidade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DP2MO',':en Porosity 2 Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DPD',':en Delta Pipe Displacement');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DPLEN',':en Drill Pipe Length :es Longitud de Tuber໚ de Perforación :ru Длина бурильной трубы :cn 钻柱长度 :pt Comprimento da Tubulação da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DPLENB',':en Drill Pipe Length @ Bottom');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DRME',':en Ream Depth :es Profundidad de Repaso :ru Глубина райберования :cn 划眼深度 :pt Profundidade Ream');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DRTM',':en Lag Depth :es Profundad de Atraso :ru Запаздывание глубины :cn 迟到井深 :pt Última Profundidade de Atraso (Lag Depth)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DRTMG',':en Lag Depth Gas :es Profundidad de Gas de Atraso :ru Запаздывание глубины газа :cn 气体迟到井深 :pt Última Profundidade de Gás (Lag Depth Gás)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DRTT',':en Lag Time :es Tiempo de Atraso :ru Время запаздывания :cn 迟到时间 :pt Último Tempo (Lag Time)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DRTTB',':en Lag Time @ Bottom');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DRTTT',':en Lag Time Theoretical :es Tiempo de Atraso Teorético :ru Теоретическое время запаздывания :cn 理论迟到时间 :pt Último Tempo Teórico (Lag Time Teórico)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DS',':en Down Strokes :es Emboladas de Bajada :ru Ход поршня вниз :cn 井口到钻头的泵冲数 :pt Movimentos para Baixo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSAM',':en Sample Depth :es Profundidad de Muestreo :ru Глубина образца :cn 取样井深 :pt Profundidade da Amostra');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSAT',':en Time to Sample :es Tiempo para Muestreo :ru Время к образцу :cn 取样等待时间 :pt Tempo até a Amostra');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSB',':en Down Strokes @ Bottom :es Down Strokes @ Botto');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSNI',':en Stands In Hole :es Parejas en el Hoyo :ru Свечи в шахте :cn 井内立柱数 :pt Seçཞs de Tubo no Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSNO',':en Total Stands :es Parejas Totales :ru Все свечи :cn 总立柱数 :pt Número Total de Seçཞs de Tubos');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSNP',':en Stands Pulled :es Parejas Sacadas :ru Вытянутые свечи :cn 已起立柱数 :pt Seçཞs de Tubo Puxadas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSR2KB',':en TRKB->SRKB Diff :es Diferencial TRKB->SRKB :ru Истинный относительный ротор->Статический относительный ротор Разн. :cn 实际补芯->静止补芯距离');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSTK',':en Delta Strokes :es Emboladas Deltas :ru Дельта перемещений :cn 泵冲数差 :pt Movimentos Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DSVM',':en S.Depth :es Profundidad de Muestreo :ru Глубина обследования :cn 测量井深 :pt Profundidade da Amostra');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DT',':en Down Time :es Tiempo de Bajada :ru Время простоя :cn 井口到钻头循环时间 :pt Tempo até em Baixo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DTB',':en Down Time @ Bottom');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DX',':en D Exponent :es Exponente D :ru D Экспонента :cn D 指数 :pt Expoente D');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('DXC',':en Dc Exponent :es Exponente DC :ru Уточнённая D Экспонента :cn Dc指数 :pt Expoente Dc');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ECD',':en ECD :ru Эквивалентная плотность циркулирующего бурового раствора :cn 当量泥浆密度');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('EHYD_ECD',':en eHydraulics ECD :es eHydraulics - ECD :pt ECD do eHydraulics');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('EHYD_SWSU',':en eHydraulics Surge/Swab Pressure :es eHydraulics - Swap/Surge :pt Pressão do Limpador do Poço/Surgência do eHydraulics');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('EMD',':en Expected Mud Density');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ETH',':en C2 :ru Этан :cn 乙烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ETH1',':en C2 Out :es C2 de Salida :ru Этан исходящий :cn 排出乙烷 :pt C2 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ETH2',':en C2 In :es C2 de Entrada :ru Этан входящий :cn 输入乙烷 :pt C2 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ETVTA',':en Expected Active PVT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ETVTT',':en Expected Trip Tank PVT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FCI1',':en Flow Controler IN GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FCI2',':en Flow Controler In GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FCO1',':en Flow Controler Out GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FCO2',':en Flow Controler Out GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FFAC',':en Formation Factor :es Factor de Formación :ru Фактор формации :cn 地层系数 :pt Fator de Formação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FFPG',':en Formation Gradient :es Gradiente de Formación :ru Градиент формации :cn 地层梯度 :pt Gradiente de Formação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FIDQC',':en FID QC Ratio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FIDQC2',':en FID 2 QC Ratio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FILES',':en Attached Files :es Archivos Adjuntos :ru Присоединённые файлы :cn 附加文件 :pt Arquivos Anexados');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FLAIRP',':en Flare Line Pressure :es Presión de Línea de Quemado :pt Linha de Pressão do Fluido');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FLC',':en Flow Line Compensation');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FLCSA',':en Flow Line Cross Section Area');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FLD',':en Flow Line Diameter');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FOBG',':en Overburden Gradient :es Gradiente de Sobrecarga :ru Градиент горного давления :cn 上覆压力梯度 :pt Gradiente de Sobrecarga');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FPPG',':en Calc Est Porosity :es Porosidad Estimada Calculada :ru Расчётная предполагаемая пористость :cn 理论空隙度 :pt Porosidade Calculada Estimado');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FSO1',':en FS Out');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FSO2',':en FS Out (IN)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FUELF',':en Fuel Flow :es Flujo de Combustible :ru Поток топлива :cn 燃料流 :pt Vazão de Combustível');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FUELL',':en Fuel Level :es Nivel de Combustible :ru Уровень топлива :cn 燃料量 :pt Nível de Combustível');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FVE',':en Theo. Displacement :es Desplazamiento Teórico :pt Deslocamento Teórico');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('FVEC',':en Acc. Theo. Displacement :es Desplazamiento Teórico Acumulado :pt Deslocamento Teórico Acumulado');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASA',':en Total Gas Sensor(avg) :es Sensor de Gas Total (Promedio) :cn 传感器气体总量(平均) :pt Sensor de Gás Total (Mຝia)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASC',':en Total Chromat Gas GCT :es Gas Total de Cromatograf໚ :ru Общий хромат газа :cn 色谱气体总量 :pt Cromatografia Gasosa Total');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASC1',':en Total Chromat Gas Out GCT :es Gas Total de Cromatograf໚ de Salida :ru Общий исходящий хромат газа :cn 总排出色谱气 :pt Cromatografia Gasosa Total de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASC2',':en Total Chromat Gas In GCT :es Gas Total de Cromatograf໚ de Entrada :ru Общий входящий хромат газа :cn 总流入色谱气 :pt Cromatografia Gasosa Total de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASC3',':en Total Chromat Gas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASN',':en Normalized Gas :es Gas Normalizado :ru Нормализованный газ :cn 校正后气体量 :pt Gás Normalizado');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASS',':en Total Gas Sensor :es Sensor de Gas Total :ru Общий датчик газа :cn 总气体传感器 :pt Sensor de Gás Total');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASS1',':en Total Gas Out :es Gas Total de Salida :cn 排出端气体总量 :pt Gás Total de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GASS2',':en Total Gas In :es Gas Total de Entrada :cn 流入端的气体总量 :pt Gás Total de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCC',':en Calc CC Gas Sensor');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF4',':en Arom/Alk');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF8',':en C1 Content');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF19',':en C1/C6');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF23',':en C1/ROP');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF24',':en C1/CO2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF25',':en C1/Benzene');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF26',':en Benzene+Toluene');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF30',':en PROETH');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF31',':en BUTISO');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCF32',':en PENTISO');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCTCT',':en GCT Cycle Time');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCTD',':en GCT Density');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCTG',':en GCT Grade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GCUT',':en Total Cuttings Gas :es Gas Total de Cortes :ru Общий бурильный газ :cn 全部岩屑内气体 :pt Gás Total dos Cuttings');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GFID',':en Calc FID Total Gas :es Gas Total FID Calculado :ru Расчётный общий газ детектора пламенной ионизации :cn 计算的火焰电离总气体量 :pt Gás Total Calc FID');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GFID2',':en CALC FID 2 TOTAL GAS');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GFO',':en Gas Flow Out :es Flujo de Salida de Gas :ru Исходящий газовый поток :cn 排出气体量 :pt Vazão de Sa do Gás');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GHW',':en Calc HW Total Gas :es Gas Total HW Calculado :ru Расчётный тепловой общий газ :cn 计算的催化燃烧总气体量 :pt Gás Total Calc. HW');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GRAV',':en Total Gravity :es Gravedad Total :cn 总重力 :pt Gravidade Total');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GSG',':en Gas SG :ru Обследованный газ :cn 气体比重 :pt Gás SG');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('GTCD',':en Calc TCD Gas Sensor');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('H',':en H :ru Водород :cn 氢气');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('H1',':en H Out :es H de Salida :ru Водород исходящий :cn 排出氢气 :pt H de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('H2',':en H In :es H de Entrada :ru Водород входящий :cn 输入氢气 :pt H de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('H2S',':en H2S :ru Сероводород :cn 硫化氢');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('H2S1',':en H2S Out :es H2S de Salida :ru Сероводород исходящий :cn 排出硫化氢 :pt H2S de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('H2S2',':en H2S In :es H2S de Entrada :ru Сероводород входящий :cn 输入硫化氢 :pt H2S de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HD',':en Hole Drag :es Arrastre del Hoyo :ru Сопротивление скважины :cn 井眼阻力 :pt Arrasto do Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HDIA',':en Hole Diameter :es Diámetro del Hoyo :ru Диаметр шахты :cn 井眼直径 :pt Diâmetro do Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HE',':en He :ru Гелий :cn 氦');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HE1',':en He Out :es He de Salida :ru Гелий исходящий :cn 排出氦气 :pt He de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HE2',':en He In :es He de Entrada :ru Гелий входящий :cn 输入氦气 :pt He de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HEAV',':en Heave :es Movimiento de Marea :ru Вертикальная качка :cn 升降');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HKLA',':en Avg Hookload :es Carga del Gancho Promedio :cn 平均钩载 :pt Carga no Gancho Mຝia');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HKLI',':en Hookload :es Carga del Gancho :ru Грузоподъёмность :cn 大钩载荷 :pt Carga no Gancho');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HKLN',':en Min Hookload :es Cargar del Gancho Mínima :pt Carga no Gancho Mínima');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HKLX',':en Max Hookload :es Cargar del Gancho Máxima :cn 最大钩载 :pt Carga no Gancho Máxima');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HMOS',':en Hook Movement Out-of-Slips :es Movimiento del Gancho Fuera de Cu༚s :ru Отсутствуют смещения движения крюка :cn 大钩移动脱离卡瓦 :pt Movimento do Gancho Fora das Cunhas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HMT',':en Hook Movement :es Movimiento del Gancho :ru Движение крюка :cn 大钩移动 :pt Movimento do Gancho');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HPOS',':en Hook Position :es Posición del Gancho :ru Позиция крюка :cn 大钩位置 :pt Posição do Gancho');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HSPD',':en Hook Speed :es Velocidad del Gancho :ru Скорость крюка :cn 大钩移动速度 :pt Velocidade do Gancho');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HUM1',':en Humidity GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HUM2',':en Humidity GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HVMP',':en Heave Rate :es Rata de Movimiento de Marea :ru Интенсивность вертикальной качки :cn 升降速率 :pt Taxa de Heave');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HVSG',':en Heave Amplitude :es Amplitud de Marea :ru Амплитуда вертикальной качки :cn 升降幅度 :pt Amplitude de Heave');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HYDC',':en Total Hydrocarbons GCT :es Hidrocarburos :ru Углеводороды :cn 烃 :pt Hidrocarbonos');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HYDC1',':en Total Hydrocarbons Out GCT :es Hidrocarburos de Salida :ru Углеводороды исходящие :cn 排出烃 :pt Hidrocarbonos de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HYDC2',':en Total Hydrocarbons In GCT :es Hidrocarburos de Entrada :ru Углеводороды входящие :cn 输入烃 :pt Hidrocarbonos de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HYDC3',':en Total Hydrocarbons MLS :es Total Hydrocarbon Chromat Out :ru Total Hydrocarbon Chromat Out :cn Total Hydrocarbon Chromat Out :pt Total Hydrocarbon Chromat Out');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HYDG',':en Hydrostatic Gradient :es Gradiente Hidrostático :ru Гидростатический градиент :cn 静水压力梯度 :pt Gradiente Hidrostático');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('HYDP',':en Hydrostatic Pressure :es Presión Hidrostática :ru Гидростатическое давление :cn 静水压力 :pt Pressão Hidrostática');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IBUT',':en iC4 :ru и-Бутан :cn 异丁烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IBUT1',':en iC4 Out :es iC4 de Salida :ru и-Бутан исходящий :cn 排出异丁烷 :pt iC4 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IBUT2',':en iC4 In :es iC4 de Entrada :ru и-Бутан входящий :cn 输入异丁烷 :pt iC4 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CB1',':en Iso-Butane Out :es Iso-Butano de Salida :pt Iso-Butano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CB2',':en Iso-Butane In :es Iso-Butano de Entrada :pt Iso-Butano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CBD',':en Delta Iso-Butane :es Delta Iso-Butano :pt Delta Iso-Butano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CR1',':en Iso-Butane Out(Raw) :es Iso-Butano de Salida (Raw) :pt Iso-Butano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CR2',':en Iso-Butane In(Raw) :es Iso-Butano de Entrada (Raw) :pt Iso-Butano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CRD',':en Delta Iso-Butane(Raw) :es Delta Iso-Butano (Raw) :pt Delta Iso-Butano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CW1',':en QC Iso-Butane Out :es Iso-Butano de Salida :cn Iso-Butane Out :pt QC Iso-Butano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CW2',':en QC Iso-Butane In :es Iso-Butano de Entrada :cn Iso-Butane In :pt QC Iso-Butano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4CWD',':en QC Delta Iso-Butane :es Delta Iso-Butano :cn Delta Iso-Butane :pt QC Delta Iso-Butano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC4FC',':en iC4 Flow corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CB1',':en Iso-Pentane Out :pt Iso-Pentano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CB2',':en Iso-Pentane In :pt Iso-Pentano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CBD',':en Delta Iso-Pentane :es Delta Iso-Pentano :pt Delta Iso-Pentano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CR1',':en Iso-Pentane Out(Raw) :es Iso-Pentano de Salida (Raw) :pt Iso-Pentano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CR2',':en Iso-Pentane In(Raw) :es Iso-Pentano de Entrada (Raw) :pt Iso-Pentano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CRD',':en Delta Iso-Pentane(Raw) :es Delta Iso-Pentano (Raw) :pt Delta Iso-Pentano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CW1',':en QC Iso-Pentane Out :es QC Iso-Pentano de Salida :cn Iso-Pentane Out :pt QC Iso-Pentano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CW2',':en QC Iso-Pentane In :es QC Iso-Pentano de Entrada :cn Iso-Pentane In :pt QC Iso-Pentano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5CWD',':en QC Delta Iso-Pentane :es QC Delta Iso-Pentano :cn Delta Iso-Pentane :pt QC Delta Iso-Pentano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IC5FC',':en iC5 Flow Corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IFA',':en Impact Force Area :es Área de Fuerza de Impacto :ru Зона ударной силы :cn 冲击压力区域 :pt Área de For๺ de Impacto');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IPEN',':en iC5 :ru и-Пентан :cn 异戊烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IPEN1',':en iC5 Out :es iC5 de Salida :ru и-Пентан исходящий :cn 排出异戊烷 :pt iC5 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('IPEN2',':en iC5 In :es iC5 de Entrada :ru и-Пентан входящий :cn 输入异戊烷 :pt iC5 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('JIF',':en Bit Impact Force :es Fuerza de Impacto de la Mecha :ru Ударная сила бура :cn 钻头冲击力 :pt For๺ de Impacto da Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('JV',':en Jet Velocity :es Velocidad de Jet :ru Скорость струи :cn 射流速度 :pt Velocidade do Jato');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('KT',':en Kick Tolerance :es Tolerancia de Kick :ru Допустимая доза выброса :cn 井涌余量 :pt Tolerância do Kick');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC1',':en Fossils 1 :es Fósiles 1 :ru Ископаемые 1 :cn 化񷼱 :pt Fosséis 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC2',':en Fossils 2 :es Fósiles 2 :ru Ископаемые 2 :cn 化񷼲 :pt Fosséis 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC3',':en Minerals 1 :es Minerales 1 :ru Минералы 1 :cn 矿񲚑 :pt Minerais 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC4',':en Minerals 2 :es Minerales 2 :ru Минералы 2 :cn 矿񲚒 :pt Minerais 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC5',':en Oil Shows 1 :es Presencia de Aceites 1 :ru Нефтепроявления 1 :cn 油气显񹎡 :pt Óleos 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC6',':en Oil Shows 2 :es Presencia de Aceites 2 :ru Нефтепроявления 2 :cn 油气显񹎢 :pt Óleos 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC7',':en Geological 1 :es Geológico 1 :ru Геологический 1 :cn 地򍊁 :pt Geológico 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC8',':en Geological 2 :es Geológico 2 :ru Геологический 2 :cn 地򍊂 :pt Geológico 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC9',':en Engineering 1 :es Ingenier໚ 1 :ru Инженерный 1 :cn 工񺂱 :pt Engenharia 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LAC10',':en Engineering 2 :es Ingenier໚ 2 :ru Инженерный 2 :cn 工񺂲 :pt Engenharia 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LFL1',':en Floresence 1 :es Fluorescencia 1 :ru Расцвет 1 :cn 荧񑒑 :pt Florescência 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LFL2',':en Floresence 2 :es Fluorescencia 2 :ru Расцвет 2 :cn 荧񑒒 :pt Florescência 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LMG',':en WL Gamma :es WL Gama :ru Гамма керна :cn 测井伽马 :pt WL Gama');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LMGA',':en WL Gamma Avg :es WL Gama Promedio :ru Гамма керна средняя :cn 测井伽马平均值 :pt WL Gama Mຝia');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LMGX',':en WL Gamma Max :es WL Gama Máxima :ru Гамма керна максимальная :cn 测井伽马最大值 :pt WL Gama Máxima');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LMR',':en WL Resistivity :es Resistividad WL :ru Сопротивление керна :cn 电阻率测井 :pt Resistividade WL');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LMS',':en WL Sonic :es WL Sónico :ru Звук керна :cn 声波测井 :pt WL Sônico');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LPOR1',':en Porosity 1 :es Porosidad 1 :ru Пористость 1 :cn 孔򖦑 :pt Porosidade 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LPOR2',':en Porosity 2 :es Porosidad 2 :ru Пористость 2 :cn 孔򖦒 :pt Porosidade 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LPRT',':en Porosity Type :es Tipo de Porosidad :ru Тип пористости :cn 孔隙类型 :pt Tipo de Porosidade');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LPT',':en Percentage Lithology :es Litolog໚ Porcentual :ru Процентная литология :cn 岩性百分数 :pt Percentual de Litologia');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LRD',':en Roundness :es Redondamiento :ru Закругленность :cn 圆度 :pt Arredondamento');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LSO',':en Sorting :es Segregación :ru Классификация :cn 分选 :pt Segregação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LSTK',':en Lag Strokes :es Emboladas de Atraso :ru Задержки хода :cn 迟到泵冲数 :pt Movimentos de Lagging');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LSTKB',':en Lag Strokes @ Bottom');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LSZ',':en Grain Size :es Tamaño de Grano :ru Размер зерна :cn 粒度 :pt Tamanho do Grão');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LTY',':en Interpreted Lithology :es Litolog໚ Interpretada :ru Интерпретированная литология :cn 岩性解释 :pt Litologia Interpretada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LVA',':en Lag Volume Adjust :es Ajuste de Volumen de Retraso :ru Настройка величины запаздывания :cn 调整延迟容积 :pt Ajuste do Volume de Lagging');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LVL',':en Lag Volume Left :es Volumen de Retraso Restante :ru Оставшаяся величина запаздывания :cn 剩余迟到体积 :pt Volume de Lagging Remanescente');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('LWEAR',':en Line Wear :es Desgaste de Línea de Perforación :ru Износ струны :cn 大绳磨损 :pt Desgaste da Linha');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MBPA',':en BH Annulus Pressure :es Presión Anular de Fondo del Pozo :pt Pressão Anular do Fundo do Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MBPI',':en BH Internal Pressure :es Presión Interna de Fondo del Pozo :pt Pressão Interna do Fundo do Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MCDA',':en Delta Mud Cond. :es Conductividad de Lodo Delta :ru Дельта состояния бурового раствора :cn 泥浆电导率差 :pt Cond. Lama Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MCIA',':en Mud Cond. In :es Conductividad de Lodo de Entrada :ru Состояние входящнго бурового раствора :cn 入口泥浆电导率 :pt Cond. Lama de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MCOA',':en Mud Cond. Out :es Conductividad de Lodo de Salida :ru Состояние исходящнго бурового раствора :cn 出口泥浆电导率 :pt Cond. Lama de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MD',':en Meters Drilled :es Metros Perforados :ru Пробурено метров :cn 进尺 :pt Metros Perfurados');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MDDA',':en Delta Mud Density :es Densidad de Lodo Delta :ru Дельта плотности бурового раствора :cn 泥浆密度差 :pt Densidade da Lama Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MDH2S',':en MDH2S :cn 泥浆罐硫化氢');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MDIA',':en Mud Density In :es Densidad de Lodo de Entrada :ru Входящая плотность бурового раствора :cn 入口泥浆密度 :pt Densidade da Lama de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MDOA',':en Mud Density Out :es Densidad de Lodo de Salida :ru Исходящая плотность бурового раствора :cn 出口泥浆密度 :pt Densidade da Lama de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('METH',':en C1 :ru Метан :cn 甲烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('METH1',':en C1 Out :es C1 de Salida :ru Метан исходящий :cn 排出甲烷 :pt C1 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('METH2',':en C1 In :es C1 de Entrada :ru Метан входящий :cn 输入甲烷 :pt C1 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFDF',':en EKDS Flow Diff :es EKDS Flujo Diferencial :pt Diff. Fluxo EKDS');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFFP',':en Fracture Gradient :es Gradiente de Fractura :ru Градиент давления гидравлического разрыва :cn 压裂梯度 :pt Gradiente de Fratura');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFGL',':en Flow G/L :es Ganancia/Pérdida de Flujo :ru Прирост/потеря потока :cn 流量增/减 :pt Ganho/Perda de Vazão');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFGLR',':en Flow G/L Rate :es Tasa de Ganancia/Pérdida de Flujo :ru Скорость прироста/потери потока :cn 流量增/减 :pt Taxa de Ganho/Perda de Vazão');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFII',':en Flow In :es Flujo de Entrada :ru Втекание :cn 流入流量 :pt Vazão de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFOI',':en Flow Out :es Flujo de Salida :ru Вытекание :cn 流出流量 :pt Vazão de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MFOP',':en Flow Out Percent :es Flujo de Salida Porcentual :ru Процент вытекания :cn 流出百分数 :pt Vazão de Sa Percentual');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MG',':en Gamma :ru Гамма :cn 伽马 :pt Gama');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MG2',':en Gamma 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MGA',':en Gamma Avg :es Gamma Promedio :ru Гамма средняя :cn 平均伽马 :pt Gama Mຝio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MGX',':en Gamma Max :es Gama Máximo :ru Гамма максимальная :cn 最大伽马 :pt Gama Máximo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MIH',':en Mud In Hole :es Lodo en el Hoyo :ru Буровой раствор в скважине :cn 井中泥浆 :pt Lama no Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MM',':en Molar Mass :es Masa Molar :ru Молярная масса :cn 摩尔质量 :pt Massa Molar');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MPHD',':en Delta Mud pH :es pH de Lodo Delta :ru Дельта водородного показателя бурового раствора :cn 泥浆pH差 :pt pH da Lama Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MPHI',':en Mud pH In :es pH de Lodo de Entrada :ru Входящий водородный показателя бурового раствора :cn 流入泥浆pH :pt pH da Lama de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MPHO',':en Mud pH Out :es pH de Lodo de Salida :ru Исходящий водородный показателя бурового раствора :cn 流出泥浆pH :pt pH da Lama de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MR',':en Resistivity Deep');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MR2',':en Resistivity Shallow');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MRD',':en Resistivity Depth Deep');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MRD2',':en Resistivity Depth Shallow');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MRD2O',':en Resistivity Shallow Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MRDO',':en Resistivity Deep Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MS',':en Sonic :es Sónico :ru Звуковой :cn 声波 :pt Sônico');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MSD',':en Sonic Depth');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MSDO',':en Sonic Offset');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MSE',':en MSE');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MSEADJ',':en MSE Adjusted :es MSE Ajustado :pt MSE Ajustado');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MSEER',':en MSE Efficiency Ratio :es Razón de Eficiencia de MSE :pt Razão de Eficiência MSE');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MTDA',':en Delta Mud Temp. :es Temperatura de Lodo Delta :ru Дельта температуры бурового раствора :cn 泥浆温度差 :pt Temperatura da Lama Delta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MTFA',':en DH Fluid Temp Ann :es Temperatura de Fluido en Fondo del Hoyo (Anular) :ru Температура скважинных флюидов в кольце :cn 井下环空泥浆温度 :pt Temper. Fluido no Fundo do Poço (Anular)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MTFP',':en DH Fluid Temp Pipe :es Temperatura de Fluido en Fondo del Hoyo (Tuber໚) :ru Температура скважинных флюидов в трубе :cn 井下钻柱内泥浆温度 :pt Temper. Fluido no Fundo do Poço (no Tubo)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MTIA',':en Mud Temp. In :es Temperatura de Lodo de Entrada :ru Температура входящего бурового раствора :cn 入口泥浆温度 :pt Temper. Lama de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MTOA',':en Mud Temp. Out :es Temperatura de Lodo de Salida :ru Температура исходящего бурового раствора :cn 出口泥浆温度 :pt Temper. Lama de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MVM',':en Mud Volume Moved :es Volumen de Lodo Movido :ru Объём перемещённого бурового раствора :cn 流动的泥浆体积 :pt Volume de Lama Movido');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('MVO',':en Mud Velocity');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2',':en N2 :ru Азот :cn 氮气');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CB1',':en Nitrogen Out :es Nitrógeno de Salida :pt Nitrogênio de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CB2',':en Nitrogen In :es Nitrógeno de Entrada :pt Nitrogênio de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CBD',':en Delta Nitrogen :es Delta Nitrógeno :pt Delta Nitrogênio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CR1',':en Nitrogen Out(Raw) :es Nitrógeno de Salida (Raw) :pt Nitrogênio de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CR2',':en Nitrogen In(Raw) :es Nitrógeno de Entrada (Raw) :pt Nitrogênio de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CRD',':en Delta Nitrogen(Raw) :es Delta Nitrógeno (Raw) :pt Delta Nitrogênio (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CW1',':en QC Nitrogen Out :es QC Nitrógeno de Salida :pt QC Nitrogênio de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CW2',':en QC Nitrogen In :es QC Nitrógeno de Entrada :pt QC Nitrogênio de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2CWD',':en QC Delta Nitrogen :es QC Delta Nitrógeno :pt QC Delta Nitrogênio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2_1',':en N2 Out :es N2 de Salida :ru Азот исходящий :cn 排出氮气 :pt N2 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('N2_2',':en N2 In :es N2 de Entrada :ru Азот входящий :cn 输入氮气 :pt N2 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NBUT',':en nC4 :ru Нео-Бутан :cn 正丁烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NBUT1',':en nC4 Out :es NC4 de Salida :ru н-Бутан исходящий :cn 排出正丁烷 :pt nC4 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NBUT2',':en nC4 In :es NC4 de Entrada :ru н-Бутан входящий :cn 输入正丁烷 :pt nC4 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CB1',':en N-Butane Out :es N-Butano de Salida :pt N-Butano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CB2',':en N-Butane In :es N-Butano de Entrada :pt N-Butano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CBD',':en Delta N-Butane :es Delta N-Butano :pt Delta N-Butano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CR1',':en N-Butane Out(Raw) :es N-Butano de Salida (Raw) :pt N-Butano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CR2',':en N-Butane In(Raw) :es N-Butano de Entrada (Raw) :pt N-Butano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CRD',':en Delta N-Butane(Raw) :es Delta N-Butano (Raw) :pt Delta N-Butano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CW1',':en N-Butane Out :es QC N-Butano de Salida :ru QC N-Butane Out :cn QC N-Butane Out :pt QC N-Butano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CW2',':en N-Butane In :es QC N-Butano de Entrada :ru QC N-Butane In :cn QC N-Butane In :pt QC N-Butano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4CWD',':en Delta N-Butane :es QC Delta N-Butano :ru QC Delta N-Butane :cn QC Delta N-Butane :pt QC Delta N-Butano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC4FC',':en nC4 Flow corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CB1',':en N-Pentane Out :pt N-Pentano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CB2',':en N-Pentane In :pt N-Pentano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CBD',':en Delta N-Pentane :es N-Pentane Delta :pt Delta N-Pentano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CR1',':en N-Pentane Out(Raw) :es N-Pentano de Salida (Raw) :pt N-Pentano de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CR2',':en N-Pentane In(Raw) :es N-Pentano de Entrada (Raw) :pt N-Pentano de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CRD',':en Delta N-Pentane(Raw) :es Delta N-Pentano (Raw) :pt Delta N-Pentano (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CW1',':en QC N-Pentane Out :es QC N-Pentano de Salida :pt QC N-Pentano de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CW2',':en QC N-Pentane In :es QC N-Pentano de Entrada :pt QC N-Pentano de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5CWD',':en QC Delta N-Pentane :es QC Delta N-Pentano :pt QC Delta N-Pentano');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NC5FC',':en nC5 Flow Corrected');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NDDMEA',':en Next Down Depth :es Próxima Profundidad de Fondo :ru Следующая глубина спуска :cn 方入 :pt Profundidade Próxima do Fundo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NPEN',':en nC5 :ru Нео-Пентан :cn 正戊烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NPEN1',':en nC5 Out :es NC5 de Salida :ru н-Пентан исходящий :cn 排出正戊烷 :pt nC5 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('NPEN2',':en nC5 In :es NC5 de Entrada :ru н-Пентан входящий :cn 输入正戊烷 :pt nC5 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('O2',':en O2 :ru Кислород :cn 氧气');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('O2_1',':en O2 Out :es O2 de Salida :ru Кислород исходящий :cn 排出氧气 :pt O2 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('O2_2',':en O2 In :es O2 de Entrada :ru Кислород входящий :cn 输入氧气 :pt O2 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('OSLIPSTAT',':en Slips Override :es Superposición de Cu༚s :cn 取消卡瓦 :pt Sobreposição da Cunha');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('OVERPULL',':en Overpull :es Sobretensión (Overpull) :ru Сверхтяга :cn 过拉 :pt Overpull (Tração Excessiva)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PFG',':en % Free Gas');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PFO',':en Pason Flow Out :es Pason - Flujo de Salida :ru Вытекающий пасон :cn Pason 流出量 :pt Fluxo Sa Pason');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PLA',':en Pressure Loss Annulus :es Pérdida de Presión Anular :ru Кольцо потери давления :cn 环空压力损失 :pt Perda de Pressão Anular');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PLB',':en Pressure Loss @ Bit :es Pérdida de Presión en la Mecha :ru Потеря давления @ бур :cn 钻头压力损失 :pt Perda de Pressão @ Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PLDS',':en Pressure Loss Pipe :es Pérdida de Presión en la Tuber໚ :ru Труба потери давления :cn 钻柱内压力损失 :pt Perda de Pressão Tubo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PLPB',':en % Pressure Loss @ Bit :es Pérdida % de Presión en la Mecha :ru % Потеря давления @ бур :cn %钻头压力损失 :pt Percentual de Perda de Pressão @ Broca');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PLSU',':en Pressure Loss @ Surface :es Pérdida de Presión en Superficie :ru Потеря давления @ поверхность :cn 地面压力损失 :pt Perda de Pressão @ Superfໜie');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PP1',':en Probe Pressure GCT OUT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PP2',':en Probe Pressure GCT IN');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PPM',':en % Mud');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PPRES',':en Pipe Pressure');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PROFILE',':en Profile Data :es Datos del Perfil :ru Данные профиля :cn 结构数据 :pt Dados de Perfil');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PRP',':en C3 :ru Пропан :cn 丙烷');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PRP1',':en C3 Out :es C3 de Salida :ru Пропан исходящий :cn 排出丙烷 :pt C3 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PRP2',':en C3 In :es C3 de Entrada :ru Пропан входящий :cn 输入丙烷 :pt C3 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PVO1',':en Pump Output 1 :es Salida de Bomba 1 :ru Мощность насоса 1 :cn 泵排򑳱 :pt Sa da Bomba 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PVO2',':en Pump Output 2 :es Salida de Bomba 2 :ru Мощность насоса 2 :cn 泵排򑳲 :pt Sa da Bomba 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PVO3',':en Pump Output 3 :es Salida de Bomba 3 :ru Мощность насоса 3 :cn 泵排򑳳 :pt Sa da Bomba 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('PVO4',':en Pump Output 4 :es Salida de Bomba 4 :ru Мощность насоса 4 :cn 泵排򑳴 :pt Sa da Bomba 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT1',':en C1/(C3+C4+C5)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT2',':en (C3+C4+C5)/C1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT3',':en C1/C2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT4',':en C1/C3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT5',':en C1/C4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT6',':en C1/C5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT7',':en C2/C3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT8',':en C6/Benz');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT9',':en C7/Tol');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT10',':en iC4/nC4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT11',':en iC5/nC5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT12',':en (C2+C3)/C1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT13',':en (C1+C3)/C2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT14',':en (C1+C2)/C3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT15',':en (C4+C5)/(C1+C2) :pt (C4+C5)/(C1 C2)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT16',':en C1/㭌');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT17',':en (C1+C4+C5)/(C1+C2+C4)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT18',':en (C1+C2)/(C3+C4+C5)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT19',':en (C4+C5)/C3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT20',':en (C2+C3+C4+C5)/δ*100.0');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT21',':en Gas/Liquid Ratio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT22',':en Aromatics/Alkanes');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT23',':en HC');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT24',':en C1/THC');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT25',':en Fluid Saturation');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT26',':en C1/C7');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT27',':en C1/C8');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT28',':en GIO3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT29',':en Fluid Saturation 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT30',':en Heavies/Lights');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT31',':en Heavies/C3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT32',':en GIO1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RAT33',':en GIO2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RATA',':en Ratio Analysis MLS');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RATA2',':en Ratio Analysis GCT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RCO',':en Riser Correction Offset :es Desvío de Corrección del Riser :pt Correção de Desvio do Riser');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RIGSTAT',':en Rig Status :es Estado del Taladro :ru Состояние бура :cn 钻机状态 :pt Situação da Plataforma');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ROPA',':en ROP Average :es ROP Promedio :ru Средняя скорость проходки :cn 平均钻速 :pt ROP Mຝio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ROPI',':en ROP :ru Скорость проходки :cn 钻速');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('ROPU',':en User ROP :es ROP del Usuario :ru Пользовательская скорость проходки :cn 用户钻速 :pt ROP do Usuário');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RPMI',':en RPM :ru Скорость вращения :cn 转速');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RPMMI',':en Mud Motor RPM :es RPM de Motor de Fondo :ru Скорость вращения забойного двигателя :cn 泥浆马达转速 :pt RPM do Motor de Lama');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RPMSI',':en Table RPM :es RPM de Mesa Rotaria :ru Скорость вращения стола :cn 转盘转速 :pt RPM da Mesa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RPMTI',':en Top Drive RPM :es RPM de Top Drive :ru Скорость верхнего привода :cn 顶驱转速 :pt RPM do Top Drive');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RPOS',':en Riser Position :es Posición del Riser :ru Позиция шахтного ствола :cn 隔水管位置 :pt Posição do Riser');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RSA',':en Running Speed :es Velocidad de Corrida :ru Рабочая скорость :cn 运行速度 :pt Velocidade de Funcionamento');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RSDN',':en Running Speed Down :es Velocidad de Corrida Bajando :ru Снижение рабочей скорости :cn 下钻速度 :pt Velocidade de Funcionamento Baixa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('RSUP',':en Running Speed Up :es Velocidad de Corrida Subiendo :ru Повышение рабочей скорости :cn 起钻速度 :pt Velocidade de Funcionamento Alta');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SAZC',':en S.Azimuth :es S. Azimut :ru Азимут обследования :cn 测量方位角 :pt S. Azimutal');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SDEN',':en Shale Density :es Densidad de Lutita :ru Плотность сланца :cn 页岩密度 :pt Densidade do Folhelho');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SDS',':en Surface Down Strokes');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SDT',':en Surface Down Time');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SFAC',':en Shale Factor :es Factor de Lutita :ru Показатель сланца :cn 页岩因数 :pt Fator do Folhelho');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SGTF',':en Gravity TF :es ToolFace Gravedad :ru Гравитационный циферблат :cn 重力工具面 :pt Gravidade TF');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SIGMA',':en Sigmalog :ru Сигма :cn 西格马');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SINC',':en S.Inclination :es Inclinación de Survey :ru Наклон обследования :cn 测量井斜 :pt Inclinação S');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SLIPSTAT',':en Slip Status :es Estado de Cu༚s :ru Состояние клиньев :cn 卡瓦状态 :pt Situação da Cunha');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SMTF',':en Magnetic TF :es ToolFace Magnético :ru Магнитный циферблат :cn 磁工具面 :pt TF Magnético');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SO2',':en SO2 :ru Диоксид серы :cn 二氧化硫');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SO2_1',':en SO2 Out :es SO2 de Salida :ru Диоксид серы исходящий :cn 排出二氧化硫 :pt SO2 de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SO2_2',':en SO2 In :es SO2 de Entrada :ru Диоксид серы входящий :cn 输入二氧化硫 :pt SO2 de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPHD',':en Delta Sulphides :es Delta Sulfatos :ru Дельта сульфиды :cn 硫化物差 :pt Delta Sulfetos');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPHI',':en Sulphides In :es Sulfuros de Entrada :ru Входящие сульфиды :cn 流入硫化物 :pt Sulfetos de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPHO',':en Sulphides Out :es Sulfuros de Salida :ru Исходящие сульфиды :cn 流出硫化物 :pt Sulfetos de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPHT',':en Total Sulphides :es Sulfuros Total :ru Общие сульфиды :cn 总硫化物量 :pt Sulfetos Totais');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPM1',':en SPM 1 :ru Число двойных ходов в минуту 1 :cn 泵򐇱');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPM2',':en SPM 2 :ru Число двойных ходов в минуту 2 :cn 泵򐇲');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPM3',':en SPM 3 :ru Число двойных ходов в минуту 3 :cn 泵򐇳');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPM4',':en SPM 4 :ru Число двойных ходов в минуту 4 :cn 泵򐇴');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPPD',':en SPP Differential :es Presión de Bomba Diferencial :pt Diferencial SPP');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SPPI',':en Standpipe Pressure :es Presión de Bomba :ru Давление на стояке :cn 立压 :pt Pressão do Stand Pipe');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAFH',':en SRA Free Hydrocarbons');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAHI',':en SRA Hydrogen Index (HI)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAHP',':en SRA Hydrocarbon Potential');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAM',':en SRA Maturity');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRANO',':en SRA Normalized Oil Content');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAOI',':en SRA OI');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAOR',':en SRA Organic Richness');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAPI',':en SRA Production Index (PI)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRAS3',':en SRA S3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SRATT',':en SRA T TEMP');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SSF',':en Stick-Slip Factor :es Factor de Stick-Slip :pt Fator de Aderência da Cunha');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SSFX',':en Stick-Slip Factor(Max) :es Factor de Stick-Slip (Máximo) :pt Fator de Aderência da Cunha (Máximo)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STDGI',':en TGD In Sample Line Delay :es Tiempo de Atraso de Gas de Muestra de Entrada :ru Время задержки газа в образце :cn 气体进入分析仪的延迟时间 :pt Retardo de Entrada da Amostra de Gás');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STDGO',':en TGD Out Sample Line Delay :es Tiempo de Atraso de Gas de Muestra de Salida :ru Время задержки газа из образца :cn 气体流出分析仪的延迟时间 :pt Retardo de Sa da Amostra de Gás');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STK1',':en Strokes 1 :es Emboladas 1 :ru Перемещения 1 :cn 泵冲񥜁 :pt Movimento 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STK2',':en Strokes 2 :es Emboladas 2 :ru Перемещения 2 :cn 泵冲񥜂 :pt Movimento 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STK3',':en Strokes 3 :es Emboladas 3 :ru Перемещения 3 :cn 泵冲񥜃 :pt Movimento 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STK4',':en Strokes 4 :es Emboladas 4 :ru Перемещения 4 :cn 泵冲񥜄 :pt Movimento 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STKC',':en Total Strokes :es Emboladas Totales :ru Всего перемещений :cn 总泵冲数 :pt Movimentos Totais');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STLEN',':en String Length :es Longitud de Tuber໚ de Perforación :ru Длина строки :cn 管串长度 :pt Comprimento da Coluna');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STLENB',':en String Length @ Bottom');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('STWT',':en String Weight :es Peso de Tuber໚ de Perforación :ru Масса строки :cn 管串长度 :pt Peso da Coluna');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SWSU',':en Surge/Swab Pressure :es Presión de Swap/Surge :ru Давление Волны/Сваба :cn 激动/抽吸压力 :pt Pressão do Limpador do Poço/Surgência');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SWSUP',':en Surge Pressure :es Presión de Surge :ru Давление волны :cn 激动压力 :pt Pressão de Surgência');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('SWSWP',':en Swab Pressure :es Presión de Swap :ru Давление сваба :cn 抽吸压力 :pt Pressão do Limpador do Poço');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TBSPM',':en Total Boost SPMs :es Emboladas Totales Booster :pt Boost SPMs Totais');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCB1',':en Toluene Out :es Tolueno de Salida :pt Tolueno de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCB2',':en Toluene In :es Tolueno de Entrada :pt Tolueno de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCBD',':en Toluene Delta :es Delta Tolueno :ru Delta Toluene :cn Delta Toluene :pt Delta Tolueno');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCDGS',':en TCD Gas Sensor :es Sensor de Gas TCD :ru Газовый сенсор :cn 热导率气体分析仪 :pt Sensor de Gás TCD');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCDQC',':en TCD QC Ratio');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCR1',':en Toluene Out(Raw) :es Tolueno de Salida (Raw) :pt Tolueno de Sa (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCR2',':en Toluene In(Raw) :es Tolueno de Entrada (Raw) :pt Tolueno de Entrada (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCRD',':en Delta Toluene(Raw) :es Delta Tolueno (Raw) :pt Delta Tolueno (Raw)');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCS',':en Total Circulating Strokes :es Emboladas de Circulación Total :ru Всего циркулирующих перемещений :cn 总循环泵冲数 :pt Movimentos Totais de Circulação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCT',':en Total Circulating Time :es Tiempo Total Circulando :ru Общее время циркуляции :cn 总循环时间 :pt Tempo Total de Circulação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCV',':en Total Circulating Volume :es Volumen Total Circulando :ru Общая величина циркуляции :cn 总循环体积量 :pt Volume Total de Circulação');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCW1',':en QC Toluene Out :es QC Tolueno de Salida :pt QC Tolueno de Sa');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCW2',':en QC Toluene In :es QC Tolueno de Entrada :pt QC Tolueno de Entrada');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TCWD',':en QC Delta Toluene :es QC Delta Tolueno :pt QC Delta Tolueno');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TDRTM',':en Time to Lag :es Tiempo para Retraso :ru Время запаздывания :cn 迟到等待时间 :pt Tempo até o Retorno');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TDRTMG',':en Time to GCT Out Lag :es Tiempo para Retraso Gas de Salida :ru Время задержки исходящего газа :cn 气体迟到等候时间 :pt Tempo até o Retorno de Sa do Gás');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TGLD',':en Total Gas Lag Depth');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TGLT',':en Total Gas Lag Time');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TGTTL',':en Total Gas Time to Lag');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('THHP',':en Total Hydraulic HP :es Poder Hidráulico Total :ru Общее гидравлическое давление :cn 总水马力 :pt HP Hidráulico Total');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('THKLI',':en Theoretical Hookload :es Carga del Gancho Teorética :ru Теоретическая грузоподъёмность :cn 理论大钩载荷 :pt Carga no Gancho Teórica');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TIME',':en Time :es Tiempo :ru Время :cn 时间 :pt Tempo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TPD',':en Pipe Displacement :es Desplazamiento de Tuber໚ :ru Замещение трубы :cn 钻柱顶替量 :pt Deslocamento do Tubo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TPL',':en Total Pressure Loss :es Pérdida de Presión Total :ru Общая потеря давления :cn 总压力损失 :pt Perda de Pressão Total');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TPS',':en Time Per Stand :es Tiempo por Pareja :cn 时间/柱 :pt Tempo por Seção de Tubo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TPSA',':en Avg Time Per Stand :es Tiempo Promedio por Pareja :cn 平均时间/柱 :pt Tempo Mຝio por Seção de Tubo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TQEI',':en Torque Electric :es Torque Elຜtrico :ru Электрический крутящий момент :cn 扭矩电量 :pt Torque Elétrico');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TQHI',':en Torque Hydraulic :es Torque Hidráulico :pt Torque Hidráulico');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TQI',':en Torque :ru Крутящий момент :cn 扭矩');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TRD',':en Trip Direction :es Dirección del Viaje :ru Направление рейса :cn 运行方向 :pt Direção da Manobra');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TRF',':en Theoretical Return Flow');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TRV',':en Theoretical Return Volume');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TSIGMA',':en T.Sigmalog');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TSPM',':en Total SPMs :es Emboladas Totales :ru Общее число двойных ходов в минуту :cn 总泵冲数');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV1',':en Pit 1 :es Tanque 1 :ru Шахта 1 :cn 泥浆񬘁 :pt Tanque 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV2',':en Pit 2 :es Tanque 2 :ru Шахта 2 :cn 泥浆񬘂 :pt Tanque 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV3',':en Pit 3 :es Tanque 3 :ru Шахта 3 :cn 泥浆񬘃 :pt Tanque 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV4',':en Pit 4 :es Tanque 4 :ru Шахта 4 :cn 泥浆񬘄 :pt Tanque 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV5',':en Pit 5 :es Tanque 5 :ru Шахта 5 :cn 泥浆񬘅 :pt Tanque 5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV6',':en Pit 6 :es Tanque 6 :ru Шахта 6 :cn 泥浆񬘆 :pt Tanque 6');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV7',':en Pit 7 :es Tanque 7 :ru Шахта 7 :cn 泥浆񬘇 :pt Tanque 7');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV8',':en Pit 8 :es Tanque 8 :ru Шахта 8 :cn 泥浆񬘈 :pt Tanque 8');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV9',':en Pit 9 :es Tanque 9 :ru Шахта 9 :cn 泥浆񬘉 :pt Tanque 9');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV10',':en Pit 10 :es Tanque 10 :ru Шахта 10 :cn 泥浆� :pt Tanque 10');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV11',':en Pit 11 :es Tanque 11 :ru Шахта 11 :cn 泥浆� :pt Tanque 11');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV12',':en Pit 12 :es Tanque 12 :ru Шахта 12 :cn 泥浆� :pt Tanque 12');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV13',':en Pit 13 :es Tanque 13 :ru Шахта 13 :cn 泥浆� :pt Tanque 13');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV14',':en Pit 14 :es Tanque 15 :ru Шахта 14 :cn 泥浆� :pt Tanque 14');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV15',':en Pit 15 :es Tanque 15 :ru Шахта 15 :cn 泥浆� :pt Tanque 15');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV16',':en Pit 16 :es Tanque 16 :ru Шахта 16 :cn 泥浆� :pt Tanque 16');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV17',':en Trip Tank 1 :es Tanque de Viaje 1 :ru Резервуар для долива :cn 起下钻泥浆罐 :pt Tanque de Manobra 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TV18',':en Trip Tank 2 :es Tanque de Viaje 2 :pt Tanque de Manobra 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVA',':en Annular Volume :es Volumen Anular :ru Кольцевой объём :cn 环空容积 :pt Volume Anular');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVAB',':en Dead Volume');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVD',':en TVD :ru Фактическая вертикальная глубина :cn 实际垂直井深');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL1',':en Pit 1 Level :pt Nível do Tanque 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL2',':en Pit 2 Level :pt Nível do Tanque 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL3',':en Pit 3 Level :pt Nível do Tanque 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL4',':en Pit 4 Level :pt Nível do Tanque 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL5',':en Pit 5 Level :pt Nível do Tanque 5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL6',':en Pit 6 Level :pt Nível do Tanque 6');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL7',':en Pit 7 Level :pt Nível do Tanque 7');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL8',':en Pit 8 Level :pt Nível do Tanque 8');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL9',':en Pit 9 Level :pt Nível do Tanque 9');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL10',':en Pit 10 Level :pt Nível do Tanque 10');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL11',':en Pit 11 Level :pt Nível do Tanque 11');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL12',':en Pit 12 Level :pt Nível do Tanque 12');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL13',':en Pit 13 Level :pt Nível do Tanque 13');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL14',':en Pit 14 Level :pt Nível do Tanque 14');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL15',':en Pit 15 Level :pt Nível do Tanque 15');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL16',':en Pit 16 Level :pt Nível do Tanque 16');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL17',':en Trip Tank 1 Level :es Nivel de Tanque 1 :pt Nível do Tanque de Manobra 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVL18',':en Trip Tank 2 Level :es Nivel de Tanque de Viaje 1 :pt Nível do Tanque de Manobra 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVP',':en Pipe Volume :es Volumen de Tuber໚ :ru Объём трубы :cn 钻杆容积 :pt Volume do Tubo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVPB',':en Pipe Volume @ Bottom');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVR',':en Riser Volume :es Volumen del Riser :ru Объём стояка :cn 隔水管容积 :pt Volume do Riser');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVT1',':en Active PVT :es PVT Activo :ru Активный общий объём :cn 在用泥浆池总量 :pt PVT Ativo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVT2',':en PVT 2 :ru Общий объём 2 :cn 泥浆池总򑳲');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVT3',':en PVT 3 :ru Общий объём 3 :cn 泥浆池总򑳳');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVT4',':en PVT 4 :ru Общий объём 4 :cn 泥浆池总򑳴');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVT5',':en Trip Tank Total :es Total de Tanque de Viaje :ru Общий резервуар для долива :cn 起下罐总量 :pt Total do Tanque de Manobra');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGL1',':en Active PVT G/L :es Ganancia/Pérdida de PVT Activo :ru Прирост/потеря активного общего объёма :cn 在用泥浆池总量增/减 :pt Ganho/Perda do PVT Ativo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGL2',':en PVT 2 G/L :es Ganancia/Pérdida de PVT 2 :ru Прирост/потеря общего объёма 2 :cn 泥浆池总򑳲 增/减 :pt Ganho/Perda do PVT 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGL3',':en PVT 3 G/L :es Ganancia/Pérdida de PVT 3 :ru Прирост/потеря общего объёма 3 :cn 泥浆池总򑳳 增/减 :pt Ganho/Perda do PVT 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGL4',':en PVT 4 G/L :es Ganancia/Pérdida de PVT 4 :ru Прирост/потеря общего объёма 4 :cn 泥浆池总򑳴增/减 :pt Ganho/Perda do PVT 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGL5',':en Triptank G/L :es Ganancia/Pérdida de Tanque de Viaje :ru Прирост/потеря в резервуаре для долива :cn 起下罐增减 :pt Ganho/Perda do Tanque de Manobra');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGLR1',':en Active PVT G/L Rate :es Rata de Ganancia/Pérdida de PVT Activo :ru Скорость прироста/потери активного общего объёма :cn 在用泥浆池总量增/减速率 :pt Taxa de Ganho/Perda do PVT Ativo');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGLR2',':en PVT2 G/L Rate :es Rata de Ganancia/Pérdida de PVT 2 :ru Скороть прироста/потери общего объёма 2 :cn 泥浆池总򑳲 增/减速率 :pt Taxa de Ganho/Perda do PVT 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGLR3',':en PVT3 G/L Rate :es Rata de Ganancia/Pérdida de PVT 3 :ru Скороть прироста/потери общего объёма 3 :cn 泥浆池总򑳳 增/减速率 :pt Taxa de Ganho/Perda do PVT 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGLR4',':en PVT4 G/L Rate :es Rata de Ganancia/Pérdida de PVT 4 :ru Скороть прироста/потери общего объёма 4 :cn 泥浆池总򑳴增/减速率 :pt Taxa de Ganho/Perda do PVT 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TVTGLR5',':en PVT5 G/L Rate :es Rata de Ganancia/Pérdida de PVT 5 :ru Скороть прироста/потери общего объёма 5 :cn 泥浆池总򑳵 增/减速率 :pt Taxa de Ganho/Perda do PVT 5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('TZ',':en Timezone :es Uso Horario :ru Часовой пояс :cn 时区 :pt Fuso Horário');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD1',':en User Defined 1 :es Definido por el Usuario 1 :ru Определено пользователем 1 :cn 已有用񢍱 :pt Definido pelo Usuário 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD2',':en User Defined 2 :es Definido por el Usuario 2 :ru Определено пользователем 2 :cn 已有用񢍲 :pt Definido pelo Usuário 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD3',':en User Defined 3 :es Definido por el Usuario 3 :ru Определено пользователем 3 :cn 已有用񢍳 :pt Definido pelo Usuário 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD4',':en User Defined 4 :es Definido por el Usuario 4 :ru Определено пользователем 4 :cn 已有用񢍴 :pt Definido pelo Usuário 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD5',':en User Defined 5 :es Definido por el Usuario 5 :ru Определено пользователем 5 :cn 已有用񢍵 :pt Definido pelo Usuário 5');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD6',':en User Defined 6 :es Definido por el Usuario 6 :ru Определено пользователем 6 :cn 已有用񢍶 :pt Definido pelo Usuário 6');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD7',':en User Defined 7 :es Definido por el Usuario 7 :ru Определено пользователем 7 :cn 已有用񢍷 :pt Definido pelo Usuário 7');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD8',':en User Defined 8 :es Definido por el Usuario 8 :ru Определено пользователем 8 :cn 已有用񢍸 :pt Definido pelo Usuário 8');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD9',':en User Defined 9 :es Definido por el Usuario 9 :ru Определено пользователем 9 :cn 已有用񢍹 :pt Definido pelo Usuário 9');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD10',':en User Defined 10 :es Definido por el Usuario 10 :ru Определено пользователем 10 :cn 已有用� :pt Definido pelo Usuário 10');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD11',':en User Defined 11 :es Definido por el Usuario 11 :ru Определено пользователем 11 :cn 已有用� :pt Definido pelo Usuário 11');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD12',':en User Defined 12 :es Definido por el Usuario 12 :ru Определено пользователем 12 :cn 已有用� :pt Definido pelo Usuário 12');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD13',':en User Defined 13 :es Definido por el Usuario 13 :ru Определено пользователем 13 :cn 已有用� :pt Definido pelo Usuário 13');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD14',':en User Defined 14 :es Definido por el Usuario 14 :ru Определено пользователем 14 :cn 已有用� :pt Definido pelo Usuário 14');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD15',':en User Defined 15 :es Definido por el Usuario 15 :ru Определено пользователем 15 :cn 已有用� :pt Definido pelo Usuário 15');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UD16',':en User Defined 16 :es Definido por el Usuario 16 :ru Определено пользователем 16 :cn 已有用� :pt Definido pelo Usuário 16');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UR1',':en User Ratio 1 :es Razón del Usuario 1 :ru Соотношение пользователя 1 :cn 用户比񳡱 :pt Razão do Usuário 1');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UR2',':en User Ratio 2 :es Razón del Usuario 2 :ru Соотношение пользователя 2 :cn 用户比񳡲 :pt Razão do Usuário 2');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UR3',':en User Ratio 3 :es Razón del Usuario 3 :ru Соотношение пользователя 3 :cn 用户比񳡳 :pt Razão do Usuário 3');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('UR4',':en User Ratio 4 :es Razón del Usuario 4 :ru Соотношение пользователя 4 :cn 用户比񳡴 :pt Razão do Usuário 4');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('VFR',':en Volumetric Flow Rate');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('WDIR',':en Wind Direction :es Dirección del Viento :ru Направление ветра :cn 风向 :pt Direção do Vento');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('WOBI',':en WOB :ru Масса на буре :cn 钻压');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('WRAT',':en Wet R MLS');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('WRAT2',':en Wet R GCT');
INSERT INTO DCDM_WellLogFieldDesc(ParamName,Descr) VALUES ('WSPD',':en Wind Speed :es Velocidad del Viento :ru Скорость ветра :cn 风速 :pt Velocidade do Vento');


--#################################################################################################
Create table DCDM_WellLog(
  Id          bigserial     NOT NULL PRIMARY KEY,                                             -- Идентификатор
  WellId      int           NOT NULL REFERENCES DCDM_Well(Id) ON DELETE CASCADE,              -- Идентификатор скважины. 
  UserId      int           NULL REFERENCES DCDM_Users(Id) ON DELETE SET NULL,                -- Ответственный за данные пользователь (если это пользователь добавляет данные)
  Mode        smallint      NOT NULL,                                                         -- Код режима работы установки (0-неизвесно 1-бурение)
  Time        timestamp     NULL DEFAULT CURRENT_TIMESTAMP,                                   -- Дата и время события
  DelayMSec   int           NULL,                                                             -- Задержка с момента предыдущего измерения этого же параметра в миллисекундах (NULL если нет такий информации)
  Depth       float8        NULL,                                                             -- Глубина
  VDepth      float8        NULL,                                                             -- Вертикальная(абсолютная) глубина
  ParamName   varchar(20)   NOT NULL,                                                         -- Название параметра
  Val         jsonb         NOT NULL,                                                         -- Значение
  isnew       boolean       NULL,                                                             -- Флаг новых данных
  isold       boolean       NULL                                                              -- флаг усторевших данных
);

GRANT ALL PRIVILEGES ON TABLE DCDM_WellLog TO dcdmsrv;

CREATE INDEX dcdm_welllog_dpidx ON dcdm_welllog (wellid,depth);                               -- Индекс для ускорения выборки по максимальной глубине (без параметров)
CREATE INDEX dcdm_welllog_baseidx ON dcdm_welllog (wellid,paramname,depth);                   -- Индекс для ускорения выборки по парметрам и глубине
CREATE INDEX dcdm_welllog_basetidx ON dcdm_welllog (wellid,paramname,time);                   -- Индекс для ускорения выборки по параметрам и времени
--#################################################################################################
Create table DCDM_CommentsAndFiles(
  Id          bigserial     NOT NULL PRIMARY KEY,                                             -- Идентификатор
  WellId      int           NOT NULL REFERENCES DCDM_Well(Id) ON DELETE CASCADE,              -- Идентификатор скважины. 
  UserId      int           NULL REFERENCES DCDM_Users(Id) ON DELETE SET NULL,                -- Пользователь добавивший комментарий
  CreateTime  timestamp     NOT NULL DEFAULT CURRENT_TIMESTAMP,                               -- Время создания/последнего изменения
  ChangeTime  timestamp     NULL,                                                             -- Время последнего изменения
  Type        smallint      NOT NULL REFERENCES DCDM_CommentsAndFilesType(Id),                -- Тип комментария
  IsIndexed   boolean       NOT NULL DEFAULT false,                                           -- Индексирована ли запись поисковой системой
  SnapDepth   float8        NULL,                                                             -- Привязка к глубине
  SnapVDepth  float8        NULL,                                                             -- Привязка к вертикальной(абсолютная) глубине
  SnapTime    timestamp     NULL,                                                             -- Привязка к дате и времи
  Comment     varchar       NULL,                                                             -- Комментарий
  OrgFileName varchar       NULL,                                                             -- Оригинальное имя файла
  FileData    bytea         NULL                                                              -- Данные файла
);

GRANT ALL PRIVILEGES ON TABLE DCDM_CommentsAndFiles TO dcdmsrv;
--#################################################################################################
--Различные параметры конфигурации
Create table DCDM_Config(
  Key        varchar(20)    NOT NULL PRIMARY KEY,                                             -- Идентификатор
  Data       varchar        NULL                                                              -- Значение параметра
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Config TO dcdmsrv;

INSERT INTO dcdm_config(key, data) VALUES ('LYTHCODEMAP','{
"13":{"f":"anhydrite.png","n":":en Anhydrite :ru Ангидрит :es Anhidrita"},
"20":{"f":"chert.png","n":":en Chert :ru Кремнистый известняк :es Esquisto"},
"18":{"f":"claystone.png","n":":en Claystone :ru Аргиллит :es Arcilla"},
"15":{"f":"coal.png","n":":en Coal :ru Уголь :es Carbón"},
"14":{"f":"conglomerate.png","n":":en Conglomerate :ru Конгломерат :es Conglomerado"},
"1":{"f":"dolomite.png","n":":en Dolomite :ru Доломит :es Dolomita"},
"19":{"f":"gypsum.png","n":":en Gypsum :ru Гипс :es Yeso"},
"255":{"f":"halite.png","n":":en Halite :ru Галит(Каменная соль) :es Hálito"},
"16":{"f":"limestone.png","n":":en Limestone :ru Известняк :es Caliza"},
"22":{"f":"metamorphic.png","n":":en Metamorphic :ru Метаморфик :es Metamórfica"},
"12":{"f":"plutonic.png","n":":en Plutonic :ru Плутоническая :es Plutónica"},
"10":{"f":"sand.png","n":":en Sand :ru Песок :es Arena"},
"17":{"f":"shale.png","n":":en Shale :ru Сланец :es Esquisto"},
"11":{"f":"siltstone.png","n":":en Siltstone :ru Алевролиты :es Piedra de limo"},
"7":{"f":"tuff.png","n":":en Tuff :ru Туф :es Toba"},
"21":{"f":"volcanic.png","n":":en Volcanic :ru Вулканический :es Volcánica"},
"9999":{"f":"unknown.png","n":":en Unknown :ru Неизвестная :es Desconocida"}}');

INSERT INTO dcdm_config(key, data) VALUES ('LYTHNAMEMAP','{
"LTY_Anhydrite":{"f":"anhydrite.png","n":":en Anhydrite :ru Ангидрит :es Anhidrita"},
"LTY_Chert":{"f":"chert.png","n":":en Chert :ru Кремнистый известняк :es Esquisto"},
"LTY_Claystone":{"f":"claystone.png","n":":en Claystone :ru Аргиллит :es Arcilla"},
"LTY_Coal":{"f":"coal.png","n":":en Coal :ru Уголь :es Carbón"},
"LTY_Conglomerate":{"f":"conglomerate.png","n":":en Conglomerate :ru Конгломерат :es Conglomerado"},
"LTY_Dolomite":{"f":"dolomite.png","n":":en Dolomite :ru Доломит :es Dolomita"},
"LTY_Gypsum":{"f":"gypsum.png","n":":en Gypsum :ru Гипс :es Yeso"},
"LTY_Halite":{"f":"halite.png","n":":en Halite :ru Галит(Каменная соль) :es Hálito"},
"LTY_Limestone":{"f":"limestone.png","n":":en Limestone :ru Известняк :es Caliza"},
"LTY_Metamorphic":{"f":"metamorphic.png","n":":en Metamorphic :ru Метаморфик :es Metamórfica"},
"LTY_Plutonic":{"f":"plutonic.png","n":":en Plutonic :ru Плутоническая :es Plutónica"},
"LTY_Sand":{"f":"sand.png","n":":en Sand :ru Песок :es Arena"},
"LTY_Shale":{"f":"shale.png","n":":en Shale :ru Сланец :es Esquisto"},
"LTY_Siltstone":{"f":"siltstone.png","n":":en Siltstone :ru Алевролиты :es Piedra de limo"},
"LTY_Tuff":{"f":"tuff.png","n":":en Tuff :ru Туф :es Toba"},
"LTY_Volcanic":{"f":"volcanic.png","n":":en Volcanic :ru Вулканический :es Volcánica"},
"LTY_Unknown":{"f":"unknown.png","n":":en Unknown :ru Неизвестная :es Desconocida"}}');

--#################################################################################################
Create table DCDM_DBMUserScripts(
  id           uuid         PRIMARY KEY DEFAULT uuid_generate_v4(),                            -- Идентификатор
  UserId       int          NULL REFERENCES DCDM_Users(Id) ON DELETE CASCADE,                 -- Пользователь добавивший комментарий
  ChangeTime   timestamp    NOT NULL DEFAULT CURRENT_TIMESTAMP,                               -- Время последнего изменения
  Name         varchar      NULL,                                                             -- Имя
  Script       json         NULL,                                                             -- Скрипт
  IsPublic     boolean      NOT NULL default false                                            -- Публичный
);

GRANT ALL PRIVILEGES ON TABLE DCDM_DBMUserScripts TO dcdmsrv;
--#################################################################################################
Create table DCDM_MLib(
  FileName    varchar(50)   NOT NULL CHECK(FileName ~ '^[a-z0-9_-]+(\.[a-z0-9_-]{1,})?$'),-- Имя файла
  Dir         varchar(50)   NULL CHECK(Dir ~ '^[a-z0-9_-]+$'),                            -- Каталог
  ChangeTime  timestamp     NULL DEFAULT CURRENT_TIMESTAMP,                               -- Время последнего изменения
  FileData    bytea         NULL,                                    	                  -- Данные файла
  PRIMARY KEY(FileName,Dir)
);
														
GRANT ALL PRIVILEGES ON TABLE DCDM_MLib TO dcdmsrv;

INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'dolomite.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'tuff.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'sand.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'siltstone.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'plutonic.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'anhydrite.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'conglomerate.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'coal.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'limestone.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'shale.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'claystone.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'gypsum.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'chert.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'volcanic.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'metamorphic.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'halite.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'unknown.png');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('rocks',NULL,'undefined.png');

INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('snd',NULL,'alert_caution.mp3');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('snd',NULL,'alert_danger.mp3');
INSERT INTO DCDM_MLib(Dir,ChangeTime,FileName) VALUES ('snd',NULL,'alert_norma.mp3');
--#################################################################################################
Create table DCDM_MarkersType(
 Id  smallint            NOT NULL PRIMARY KEY,                     -- Идентификатор
 Name varchar            NOT NULL UNIQUE                           -- Название (мультиязычная строка)
);
GRANT ALL PRIVILEGES ON TABLE DCDM_MarkersType TO dcdmsrv;

INSERT INTO DCDM_MarkersType(Id,Name) VALUES ( 1,':en Marker 1 :ru Маркер 1 :es Marcador 1');
INSERT INTO DCDM_MarkersType(Id,Name) VALUES ( 2,':en Marker 2 :ru Маркер 2 :es Marcador 2');
INSERT INTO DCDM_MarkersType(Id,Name) VALUES ( 3,':en Marker 3 :ru Маркер 3 :es Marcador 3');
INSERT INTO DCDM_MarkersType(Id,Name) VALUES ( 4,':en Marker 4 :ru Маркер 4 :es Marcador 4');
INSERT INTO DCDM_MarkersType(Id,Name) VALUES ( 5,':en Marker 5 :ru Маркер 5 :es Marcador 5');
--#################################################################################################
CREATE TYPE floatrange AS RANGE (subtype=float8,subtype_diff=float8mi);


Create table DCDM_Markers(
  Id          bigserial     NOT NULL PRIMARY KEY,                                             -- Идентификатор
  WellId      int           NOT NULL REFERENCES DCDM_Well(Id) ON DELETE CASCADE,              -- Идентификатор скважины. 
  UserId      int           NULL REFERENCES DCDM_Users(Id) ON DELETE SET NULL,                -- Пользователь добавивший маркер
  CreateTime  timestamp     NOT NULL DEFAULT CURRENT_TIMESTAMP,                               -- Время создания
  ChangeTime  timestamp     NULL,                                                             -- Время последнего изменения
  Type        smallint      NOT NULL REFERENCES DCDM_MarkersType(Id),                         -- Тип маркера
  Depth       floatrange    NULL,                                                             -- Диапазон привязки к глубине
  VDepth      floatrange    NULL,                                                             -- Диапазон привязки к вертикальной(абсолютная) глубине
  Time        tsrange       NULL,                                                             -- Диапазон привязки к дате и времи
  Code        int           NOT NULL                                                          -- Код маркера
);

GRANT ALL PRIVILEGES ON TABLE DCDM_Markers TO dcdmsrv;
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--#################################################################################################
--GRANT USAGE, SELECT ON ALL SEQUENCES IN SCHEMA public TO dcdmsrv;
--#################################################################################################
CREATE OR REPLACE FUNCTION GetUserList(i_isadmin bool)
RETURNS TABLE(id dcdm_users.id%TYPE,login dcdm_users.login%TYPE,name dcdm_users.name%TYPE,
			  permission dcdm_users.permission%TYPE,isdisabled dcdm_users.isdisabled%TYPE,
           latestuse dcdm_users.latestuse%TYPE,acgroupname dcdm_accessgroups.name%TYPE)
IMMUTABLE LANGUAGE 'plpgsql' AS $$
BEGIN
 RETURN QUERY select DCDM_Users.Id,
                     DCDM_Users.Login,
					 DCDM_Users.Name,
					 DCDM_Users.Permission,
					 DCDM_Users.IsDisabled,
					 DCDM_Users.LatestUse,
					 dcdm_accessgroups.name as acgroupname
		      from DCDM_Users
			  left outer join dcdm_accessgroups on dcdm_accessgroups.id=dcdm_users.acgroup
			  where i_isadmin=true OR (i_isadmin=false AND POSITION('A' IN DCDM_Users.Permission)=0 )
           order by DCDM_Users.Id;
END
$$;

GRANT EXECUTE ON FUNCTION  GetUserList(bool) TO dcdmsrv;
--#################################################################################################
CREATE OR REPLACE FUNCTION GetUserItem(i_isadmin bool,i_id dcdm_users.id%TYPE)
RETURNS TABLE(id dcdm_users.id%TYPE,login dcdm_users.login%TYPE,name dcdm_users.name%TYPE,
			  permission dcdm_users.permission%TYPE,lang dcdm_users.lang%TYPE,isdisabled dcdm_users.isdisabled%TYPE,
			  acgroup dcdm_users.acgroup%TYPE,latestuse dcdm_users.latestuse%TYPE)
IMMUTABLE LANGUAGE 'plpgsql' AS $$
BEGIN
 RETURN QUERY select DCDM_Users.Id,
                     DCDM_Users.Login,
					 DCDM_Users.Name,
					 DCDM_Users.Permission,
					 DCDM_Users.Lang,
					 DCDM_Users.IsDisabled,
					 DCDM_Users.AcGroup,
					 DCDM_Users.LatestUse
		      from DCDM_Users 
			  where DCDM_Users.Id=i_id AND (i_isadmin=true OR (i_isadmin=false AND POSITION('A' IN DCDM_Users.Permission)=0));
END
$$;

GRANT EXECUTE ON FUNCTION  GetUserItem(bool,dcdm_users.id%TYPE) TO dcdmsrv;
--#################################################################################################
CREATE OR REPLACE FUNCTION AddChangeUserData(i_isadmin bool,i_id dcdm_users.id%TYPE,i_login dcdm_users.login%TYPE,i_name dcdm_users.name%TYPE,
							 i_perm dcdm_users.permission%TYPE,i_lang dcdm_users.lang%TYPE,i_hash dcdm_users.passhash%TYPE,
							 i_salt dcdm_users.salt%TYPE,i_disflg dcdm_users.isdisabled%TYPE,i_acgroup dcdm_users.acgroup%TYPE)
RETURNS TABLE(id dcdm_users.id%TYPE,login dcdm_users.login%TYPE,name dcdm_users.name%TYPE,
			  permission dcdm_users.permission%TYPE,isdisabled dcdm_users.isdisabled%TYPE,
           latestuse dcdm_users.latestuse%TYPE,acgroupname dcdm_accessgroups.name%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 -- Проверим корректны ли права редактируемого/добавляемого пользователя
 if POSITION('A' IN i_perm)>0 AND i_isadmin=false then RAISE EXCEPTION 'You have not enough permission to set this privileges' USING HINT='DCDM', DETAIL='PermError';end if;
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_users(login,name,passhash,salt,permission,lang,isdisabled,acgroup) 
  VALUES (i_login,i_name,i_hash,i_salt,i_perm,i_lang,i_disflg,i_acgroup);
  select lastval() into i_id;
 else 
  -- Редактирование 
  if length(coalesce(i_hash,''))>0 AND length(coalesce(i_salt,''))>0 
    then UPDATE dcdm_users SET passhash=i_hash, salt=i_salt WHERE dcdm_users.id=i_id;
  end if;			   
  UPDATE dcdm_users SET login=i_login, name=i_name, permission=i_perm, 
						lang=i_lang, isdisabled=i_disflg, acgroup=case when i_acgroup=0	then null else i_acgroup end
  WHERE dcdm_users.id=i_id;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY select dcdm_users.id,dcdm_users.login,dcdm_users.name,dcdm_users.permission,dcdm_users.isdisabled,dcdm_users.latestuse,dcdm_accessgroups.name as acgroupname
  from dcdm_users left outer join dcdm_accessgroups on dcdm_accessgroups.id=dcdm_users.acgroup
  where dcdm_users.id=i_id;

 EXCEPTION WHEN unique_violation then RAISE EXCEPTION 'Login must be unique' USING HINT='DCDM', DETAIL='LoginError';
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeUserData(bool,dcdm_users.id%TYPE,dcdm_users.login%TYPE,dcdm_users.name%TYPE,dcdm_users.permission%TYPE,
											 dcdm_users.lang%TYPE,dcdm_users.passhash%TYPE,dcdm_users.salt%TYPE,dcdm_users.isdisabled%TYPE,
											 dcdm_users.acgroup%TYPE) TO dcdmsrv;
--#################################################################################################
CREATE OR REPLACE FUNCTION RemoveUserItem(i_isadmin bool,i_id dcdm_users.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_users WHERE dcdm_users.id=i_id AND ((POSITION('A' IN dcdm_users.permission)>0 AND i_isadmin=true) OR POSITION('A' IN dcdm_users.permission)=0);
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveUserItem(bool,dcdm_users.id%TYPE) TO dcdmsrv;
--#################################################################################################
SET plpgsql.extra_warnings TO 'shadowed_variables';
CREATE OR REPLACE FUNCTION AddChangeVarData(
 i_orgname dcdm_vars.sysname%TYPE,
 i_name dcdm_vars.sysname%TYPE,
 i_type dcdm_vars.type%TYPE,
 i_init dcdm_vars.initexpr%TYPE,
 i_calc dcdm_vars.calcexpr%TYPE,
 i_tlrdvg dcdm_vars.tlrdvg%TYPE,
 i_loggedpar dcdm_vars.loggedpar%TYPE,
 i_isext dcdm_vars.isexternal%TYPE,
 i_timeout dcdm_vars.timeout%TYPE,
 i_avgmethod dcdm_vars.avgmethod%TYPE,
 i_desc dcdm_vars.description%TYPE,
 i_lag dcdm_vars.lag%TYPE,
 i_group dcdm_vars.groupid%TYPE,
 i_filter dcdm_vars.filter%TYPE
)
RETURNS TABLE(
 sysname dcdm_vars.sysname%TYPE,
 type dcdm_vartype.name%TYPE,	
 groupid dcdm_vars.groupid%TYPE,
 groupname dcdm_vargroups.name%TYPE,
 description dcdm_vars.description%TYPE,
 loggedpar dcdm_vars.loggedpar%TYPE,
 isexternal dcdm_vars.isexternal%TYPE,
 avgmethod dcdm_vars.avgmethod%TYPE,
 sortpos dcdm_vars.sortpos%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE 
 sp dcdm_vars.sortpos%TYPE;
BEGIN
 IF i_orgname IS NULL then
  -- Добавление новой переменной 
  -- Получим максимальное значение позиции сортировки
  select COALESCE(MAX(dcdm_vars.sortpos),0)+1 from dcdm_vars into sp;
  -- Добавим переменную
  INSERT INTO dcdm_vars(sysname, groupid, type, initexpr, calcexpr, tlrdvg, loggedpar, isexternal, timeout, avgmethod, description, sortpos,lag,filter)
  VALUES (i_name,i_group,i_type,i_init,i_calc,i_tlrdvg,i_loggedpar,i_isext,i_timeout,i_avgmethod,i_desc,sp,i_lag,i_filter); 
 else
  -- редактирование
  UPDATE dcdm_vars SET sysname=i_name, groupid=i_group, type=i_type, initexpr=i_init, calcexpr=i_calc, 
   tlrdvg=i_tlrdvg, loggedpar=i_loggedpar, isexternal=i_isext, timeout=i_timeout, avgmethod=i_avgmethod, description=i_desc,
   lag=i_lag,filter=i_filter
  WHERE dcdm_vars.sysname=i_orgname;  
 end if;
 
 RETURN QUERY select dcdm_vars.sysname,dcdm_vartype.name as type,dcdm_vars.groupid,dcdm_vargroups.name as groupname ,dcdm_vars.description,dcdm_vars.loggedpar,dcdm_vars.isexternal,dcdm_vars.avgmethod,dcdm_vars.sortpos 
  from dcdm_vars 
  inner join dcdm_vartype on dcdm_vartype.id=dcdm_vars.type
  left outer join dcdm_vargroups on dcdm_vargroups.id=dcdm_vars.groupid
  where dcdm_vars.sysname=i_name;

 EXCEPTION 
  WHEN unique_violation then RAISE EXCEPTION 'Name must be unique' USING HINT='DCDM', DETAIL='VarNameError';
  WHEN check_violation then RAISE EXCEPTION 'Invalid character' USING HINT='DCDM', DETAIL='VarInvalidChar';
 
END
$$;

GRANT EXECUTE ON FUNCTION  AddChangeVarData(
 dcdm_vars.sysname%TYPE,dcdm_vars.sysname%TYPE,dcdm_vars.type%TYPE,dcdm_vars.initexpr%TYPE,dcdm_vars.calcexpr%TYPE,
 dcdm_vars.tlrdvg%TYPE,dcdm_vars.loggedpar%TYPE,dcdm_vars.isexternal%TYPE,dcdm_vars.timeout%TYPE,
 dcdm_vars.avgmethod%TYPE,dcdm_vars.description%TYPE,dcdm_vars.lag%TYPE,dcdm_vars.groupid%TYPE,dcdm_vars.filter%TYPE)
TO dcdmsrv;
--#################################################################################################					 
/*
CREATE OR REPLACE FUNCTION GetVarItem(i_sysname dcdm_vars.sysname%TYPE)
RETURNS TABLE(sysname dcdm_vars.sysname%TYPE, type dcdm_vars.type%TYPE, initexpr dcdm_vars.initexpr%TYPE,calcexpr dcdm_vars.calcexpr%TYPE,
			  tlrdvg dcdm_vars.tlrdvg%TYPE, loggedpar dcdm_vars.loggedpar%TYPE, isexternal dcdm_vars.isexternal%TYPE, timeout dcdm_vars.timeout%TYPE, avgmethod dcdm_vars.avgmethod%TYPE,description dcdm_vars.description%TYPE)
IMMUTABLE LANGUAGE 'plpgsql' AS $$
BEGIN
 RETURN QUERY SELECT dcdm_vars.sysname, dcdm_vars.type, dcdm_vars.initexpr, dcdm_vars.calcexpr, dcdm_vars.tlrdvg, dcdm_vars.loggedpar, dcdm_vars.isexternal, dcdm_vars.timeout, dcdm_vars.avgmethod, dcdm_vars.description
  FROM dcdm_vars
  WHERE dcdm_vars.sysname=i_sysname;
END
$$;

GRANT EXECUTE ON FUNCTION  GetVarItem(dcdm_vars.sysname%TYPE) TO dcdmsrv;
*/
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveVarItem(i_sysname dcdm_vars.sysname%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_vars WHERE dcdm_vars.sysname=i_sysname;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveVarItem(dcdm_vars.sysname%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION SortVar(i_sysname dcdm_vars.sysname%TYPE,i_direction char(1))
RETURNS TABLE(
 sysname dcdm_vars.sysname%TYPE,
 type dcdm_vartype.name%TYPE,	
 groupid dcdm_vars.groupid%TYPE,
 groupname dcdm_vargroups.name%TYPE,
 description dcdm_vars.description%TYPE,
 loggedpar dcdm_vars.loggedpar%TYPE,
 isexternal dcdm_vars.isexternal%TYPE,
 avgmethod dcdm_vars.avgmethod%TYPE,
 sortpos dcdm_vars.sortpos%TYPE,
 filter dcdm_vars.filter%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 v_pos dcdm_vars.sortpos%TYPE;
 v_newpos dcdm_vars.sortpos%TYPE;
 v_psysname dcdm_vars.sysname%TYPE;
BEGIN
 select dcdm_vars.sortpos from dcdm_vars where dcdm_vars.sysname=i_sysname into v_pos;

 if i_direction='D' then
  select dcdm_vars.sortpos,dcdm_vars.sysname into v_newpos,v_psysname from dcdm_vars where dcdm_vars.sortpos>v_pos order by dcdm_vars.sortpos limit 1;
 elsif i_direction='U' then
  select dcdm_vars.sortpos,dcdm_vars.sysname into v_newpos,v_psysname from dcdm_vars where dcdm_vars.sortpos<v_pos order by dcdm_vars.sortpos desc limit 1;
 else
  v_newpos:=null;
  v_psysname:=null;
 end if; 
																					 
																					 
 if v_psysname IS NOT NULL AND v_newpos IS NOT NULL then
  UPDATE dcdm_vars SET sortpos=v_newpos WHERE dcdm_vars.sysname=i_sysname;
  UPDATE dcdm_vars SET sortpos=v_pos WHERE dcdm_vars.sysname=v_psysname;
 else 
  RAISE EXCEPTION 'Unknown sort direction' USING HINT='DCDM', DETAIL='VarSortError';
 end if; 

 RETURN QUERY select dcdm_vars.sysname,dcdm_vartype.name as type,dcdm_vars.groupid,dcdm_vargroups.name as groupname,
      dcdm_vars.description,dcdm_vars.loggedpar,dcdm_vars.isexternal,dcdm_vars.avgmethod,dcdm_vars.sortpos,dcdm_vars.filter 
  from dcdm_vars 
  inner join dcdm_vartype on dcdm_vartype.id=dcdm_vars.type
  left outer join dcdm_vargroups on dcdm_vargroups.id=dcdm_vars.groupid
  where dcdm_vars.sysname in(i_sysname,v_psysname);
END
$$;

GRANT EXECUTE ON FUNCTION  SortVar(dcdm_vars.sysname%TYPE,char) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeProfileData(i_id dcdm_profiles.id%TYPE,i_name dcdm_profiles.name%TYPE,i_desc dcdm_profiles.description%TYPE)
RETURNS TABLE(id dcdm_profiles.id%TYPE,name dcdm_profiles.name%TYPE,description dcdm_profiles.description%TYPE,isactive dcdm_profiles.isactive%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_profiles(name, description) VALUES (i_name,i_desc);
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_profiles SET name=i_name, description=i_desc WHERE dcdm_profiles.id=i_id;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY SELECT dcdm_profiles.id, dcdm_profiles.name, dcdm_profiles.description, dcdm_profiles.isactive FROM dcdm_profiles where dcdm_profiles.id=i_id;
 
 EXCEPTION WHEN unique_violation then RAISE EXCEPTION 'Name must be unique' USING HINT='DCDM', DETAIL='PrfNameError';
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeProfileData(dcdm_profiles.id%TYPE,dcdm_profiles.name%TYPE,dcdm_profiles.description%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveProfileItem(i_id dcdm_profiles.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_profiles WHERE dcdm_profiles.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveProfileItem(dcdm_profiles.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION ProfileActivate(i_id dcdm_profiles.id%TYPE)
RETURNS dcdm_profiles.id%TYPE VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 -- Сбросим флаг активации со всех профилей
 UPDATE dcdm_profiles SET isactive=false WHERE isactive=true;
 -- Установим флаг на нужный профиль
 UPDATE dcdm_profiles SET isactive=true	WHERE id=i_id;
 RETURN i_id;
END
$$;
GRANT EXECUTE ON FUNCTION  ProfileActivate(dcdm_profiles.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeIPointData(i_id dcdm_ipoints.id%TYPE,i_profileid dcdm_ipoints.profileid%TYPE,i_pointtype dcdm_ipoints.pointtype%TYPE,
							 i_name dcdm_ipoints.name%TYPE,i_config dcdm_ipoints.config%TYPE)
RETURNS TABLE(id dcdm_ipoints.id%TYPE,profileid dcdm_ipoints.profileid%TYPE,pointtype dcdm_ipoints.pointtype%TYPE,name dcdm_ipoints.name%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_ipoints(profileid, pointtype, name, config) VALUES (i_profileid,i_pointtype,i_name,i_config);  
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_ipoints SET pointtype=i_pointtype, name=i_name, config=i_config
  WHERE dcdm_ipoints.id=i_id AND dcdm_ipoints.profileid=i_profileid;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY SELECT dcdm_ipoints.id, dcdm_ipoints.profileid, dcdm_ipoints.pointtype, dcdm_ipoints.name
   FROM dcdm_ipoints 
   WHERE dcdm_ipoints.id=i_id;
 
 EXCEPTION WHEN unique_violation then RAISE EXCEPTION 'Name must be unique' USING HINT='DCDM', DETAIL='IPointNameError';
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeIPointData(dcdm_ipoints.id%TYPE,dcdm_ipoints.profileid%TYPE,dcdm_ipoints.pointtype%TYPE,dcdm_ipoints.name%TYPE,dcdm_ipoints.config%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveIPointItem(i_id dcdm_ipoints.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_ipoints WHERE dcdm_ipoints.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveIPointItem(dcdm_ipoints.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeRefData(i_id dcdm_referenceslist.id%TYPE,i_name dcdm_referenceslist.name%TYPE,i_sysname dcdm_referenceslist.sysname%TYPE,
							 i_iswelldep dcdm_referenceslist.iswelldep%TYPE)
RETURNS TABLE(id dcdm_referenceslist.id%TYPE,name dcdm_referenceslist.name%TYPE,sysname dcdm_referenceslist.sysname%TYPE,iswelldep dcdm_referenceslist.iswelldep%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_referenceslist(name, sysname, iswelldep) VALUES (i_name,i_sysname,i_iswelldep);  
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_referenceslist SET name=i_name, sysname=i_sysname, iswelldep=i_iswelldep  WHERE dcdm_referenceslist.id=i_id;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY SELECT dcdm_referenceslist.id, dcdm_referenceslist.name, dcdm_referenceslist.sysname, dcdm_referenceslist.iswelldep
   FROM dcdm_referenceslist 
   WHERE dcdm_referenceslist.id=i_id;
 
 EXCEPTION WHEN unique_violation then RAISE EXCEPTION 'SysName must be unique' USING HINT='DCDM', DETAIL='SysnameError';
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeRefData(dcdm_referenceslist.id%TYPE,dcdm_referenceslist.name%TYPE,dcdm_referenceslist.sysname%TYPE,dcdm_referenceslist.iswelldep%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveRefItem(i_id dcdm_referenceslist.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_referenceslist WHERE dcdm_referenceslist.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveRefItem(dcdm_referenceslist.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeRefColData(i_id dcdm_referencecol.id%TYPE,i_reflistid dcdm_referencecol.reflistid%TYPE,i_colname dcdm_referencecol.colname%TYPE,i_sysname dcdm_referencecol.sysname%TYPE,
							 i_type dcdm_referencecol.type%TYPE,i_initexpr dcdm_referencecol.initexpr%TYPE)
RETURNS TABLE(id dcdm_referencecol.id%TYPE,reflistid dcdm_referencecol.reflistid%TYPE,colname dcdm_referencecol.colname%TYPE,sysname dcdm_referencecol.sysname%TYPE,type dcdm_vartype.name%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_referencecol(reflistid, colname, sysname, type, initexpr) VALUES (i_reflistid,i_colname,i_sysname,i_type,i_initexpr);  
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_referencecol SET reflistid=i_reflistid, colname=i_colname, sysname=i_sysname, type=i_type, initexpr=i_initexpr  WHERE dcdm_referencecol.id=i_id;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY SELECT dcdm_referencecol.id, dcdm_referencecol.reflistid, dcdm_referencecol.colname, dcdm_referencecol.sysname, dcdm_vartype.name
   FROM dcdm_referencecol 
   inner join dcdm_vartype on dcdm_vartype.id=dcdm_referencecol.type
   WHERE dcdm_referencecol.id=i_id;
 
 EXCEPTION WHEN unique_violation then RAISE EXCEPTION 'SysName must be unique' USING HINT='DCDM', DETAIL='SysnameError';
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeRefColData(dcdm_referencecol.id%TYPE,dcdm_referencecol.reflistid%TYPE,dcdm_referencecol.colname%TYPE,dcdm_referencecol.sysname%TYPE,dcdm_referencecol.type%TYPE,dcdm_referencecol.initexpr%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveRefColItem(i_id dcdm_referencecol.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_referencecol WHERE dcdm_referencecol.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveRefColItem(dcdm_referencecol.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeWellData(i_id dcdm_well.id%TYPE,i_name dcdm_well.name%TYPE,i_lat dcdm_well.latitude%TYPE,i_lon dcdm_well.longitude%TYPE)
RETURNS TABLE(id dcdm_well.id%TYPE,name dcdm_well.name%TYPE,latitude dcdm_well.latitude%TYPE,longitude dcdm_well.longitude%TYPE,crdate dcdm_well.crdate%TYPE,isactive dcdm_well.isactive%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_well(name, latitude, longitude) VALUES (i_name,i_lat,i_lon);  
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_well SET name=i_name, latitude=i_lat, longitude=i_lon  WHERE dcdm_well.id=i_id;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY SELECT dcdm_well.id, dcdm_well.name, dcdm_well.latitude, dcdm_well.longitude,dcdm_well.crdate, dcdm_well.isactive
   FROM dcdm_well 
   WHERE dcdm_well.id=i_id;
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeWellData(dcdm_well.id%TYPE,dcdm_well.name%TYPE,dcdm_well.latitude%TYPE,dcdm_well.longitude%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveWellItem(i_id dcdm_well.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_well WHERE dcdm_well.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveWellItem(dcdm_well.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION ActivateWellItem(i_id dcdm_well.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 UPDATE dcdm_well SET isactive=false;
 UPDATE dcdm_well SET isactive=true WHERE dcdm_well.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  ActivateWellItem(dcdm_well.id%TYPE) TO dcdmsrv;

--#################################################################################################					 
/*
Сохранение информации в справочнике
 $1 - Идентификатор записи или 0 в случае создания новой
 $2 - Идентификатор справочника
 $3 - Идентификатор скважины для оперативных справочников или NULL для общих
 $4 - Префикс группы данных(мнемоническое имя) или NULL есди не требуется
 $5 - Данные справочника (json объект)
*/

CREATE OR REPLACE FUNCTION AddChangeRefDataItem(i_id dcdm_referencedata.id%TYPE,i_reflistid dcdm_referencedata.reflistid%TYPE,
												 i_wellid dcdm_referencedata.wellid%TYPE,i_grpname dcdm_referencedata.grpname%TYPE,
											     i_val dcdm_referencedata.val%TYPE)
RETURNS TABLE(id dcdm_referencedata.id%TYPE,wellid dcdm_referencedata.wellid%TYPE,reflistid dcdm_referencedata.reflistid%TYPE,
			  subid dcdm_referencedata.subid%TYPE,grpname dcdm_referencedata.grpname%TYPE,val dcdm_referencedata.val%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 v_subid int;
BEGIN
 -- Сперва проверим соответствует ли тип справочкика заявленым параметрам
 IF EXISTS (SELECT * FROM dcdm_referenceslist where dcdm_referenceslist.id=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referenceslist.iswelldep=true) OR (i_wellid IS NULL AND dcdm_referenceslist.iswelldep=false))) THEN
  -- TODO: Сделать проверку корректности json объектов
  if i_id=0 then 
   -- Добавление
   -- Найдем максимальное значение для SubId указанного справочника																													   
   SELECT coalesce(MAX(dcdm_referencedata.subid),0)+1 FROM dcdm_referencedata where dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid) OR (i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL)) into v_subid;
   -- Добавим запись
   INSERT INTO dcdm_referencedata(wellid, reflistid, subid, grpname, val) VALUES (i_wellid,i_reflistid,v_subid,i_grpname,i_val);																																		 
   -- Узнаем идентификатор добавленной записи
   select lastval() into i_id;																																		   
  else 			
   -- Обновление		
   UPDATE dcdm_referencedata SET wellid=i_wellid, reflistid=i_reflistid, grpname=i_grpname, val=i_val WHERE dcdm_referencedata.id=i_id; 
  end if;
  RETURN QUERY SELECT dcdm_referencedata.id, dcdm_referencedata.wellid, dcdm_referencedata.reflistid, dcdm_referencedata.subid, dcdm_referencedata.grpname, dcdm_referencedata.val
   FROM dcdm_referencedata where dcdm_referencedata.id=i_id;
 ELSE																													   
  RAISE EXCEPTION 'Invalid reference or well id' USING HINT='DCDM', DETAIL='RefIdOrWellIdErr';																													   
 END IF;																													   
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeRefDataItem(dcdm_referencedata.id%TYPE,dcdm_referencedata.reflistid%TYPE,dcdm_referencedata.wellid%TYPE,dcdm_referencedata.grpname%TYPE,dcdm_referencedata.val%TYPE) TO dcdmsrv;
--#################################################################################################					 
-- 2 параметра
CREATE OR REPLACE FUNCTION GetRefDataItem(i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE)
RETURNS TABLE(id dcdm_referencedata.id%TYPE,wellid dcdm_referencedata.wellid%TYPE,reflistid dcdm_referencedata.reflistid%TYPE,
			  subid dcdm_referencedata.subid%TYPE,grpname dcdm_referencedata.grpname%TYPE,val dcdm_referencedata.val%TYPE) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
BEGIN
 IF EXISTS (SELECT * FROM dcdm_referenceslist where dcdm_referenceslist.id=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referenceslist.iswelldep=true) OR (i_wellid IS NULL AND dcdm_referenceslist.iswelldep=false))) THEN
  RETURN QUERY select dcdm_referencedata.id, dcdm_referencedata.wellid, dcdm_referencedata.reflistid, dcdm_referencedata.subid, dcdm_referencedata.grpname, dcdm_referencedata.val
  FROM dcdm_referencedata 
  where dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid))
  ORDER BY dcdm_referencedata.subid,dcdm_referencedata.grpname;
 ELSE
  RAISE EXCEPTION 'Invalid reference or well id' USING HINT='DCDM', DETAIL='RefIdOrWellIdErr';																													   
 END IF;
END
$$;
GRANT EXECUTE ON FUNCTION  GetRefDataItem(i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE) TO dcdmsrv;
--#################################################################################################					 
-- 3 параметра
CREATE OR REPLACE FUNCTION GetRefDataItem(i_id dcdm_referencedata.id%TYPE,i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE)
RETURNS TABLE(id dcdm_referencedata.id%TYPE,wellid dcdm_referencedata.wellid%TYPE,reflistid dcdm_referencedata.reflistid%TYPE,
			  subid dcdm_referencedata.subid%TYPE,grpname dcdm_referencedata.grpname%TYPE,val dcdm_referencedata.val%TYPE) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
BEGIN
 IF EXISTS (SELECT * FROM dcdm_referenceslist where dcdm_referenceslist.id=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referenceslist.iswelldep=true) OR (i_wellid IS NULL AND dcdm_referenceslist.iswelldep=false))) THEN
  RETURN QUERY select dcdm_referencedata.id, dcdm_referencedata.wellid, dcdm_referencedata.reflistid, dcdm_referencedata.subid, dcdm_referencedata.grpname, dcdm_referencedata.val
  FROM dcdm_referencedata 
  where dcdm_referencedata.id=i_id AND dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid));
 ELSE
  RAISE EXCEPTION 'Invalid reference or well id' USING HINT='DCDM', DETAIL='RefIdOrWellIdErr';																													   
 END IF;
END
$$;
GRANT EXECUTE ON FUNCTION  GetRefDataItem(dcdm_referencedata.id%TYPE,dcdm_referencedata.reflistid%TYPE,dcdm_referencedata.wellid%TYPE) TO dcdmsrv;
--#################################################################################################	
CREATE OR REPLACE FUNCTION RemoveRefDataItem(i_id dcdm_referencedata.id%TYPE,i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 -- Сперва проверим соответствует ли тип справочкика заявленым параметрам
 IF EXISTS (SELECT * FROM dcdm_referenceslist where dcdm_referenceslist.id=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referenceslist.iswelldep=true) OR (i_wellid IS NULL AND dcdm_referenceslist.iswelldep=false))) THEN
  DELETE FROM dcdm_referencedata 
  where dcdm_referencedata.id=i_id AND dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid));  
  GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
  RETURN ResulrCount;
 ELSE RAISE EXCEPTION 'Invalid reference or well id' USING HINT='DCDM', DETAIL='RefIdOrWellIdErr';																													   
 END IF;																													   
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveRefDataItem(i_id dcdm_referencedata.id%TYPE,i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION ReposRefDataItem(i_id dcdm_referencedata.id%TYPE,i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE,i_newsubid dcdm_referencedata.subid%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 v_oldsubid int;
 ResulrCount int;
 curs cursor for select dcdm_referencedata.id--,dcdm_referencedata.subid 
                   from dcdm_referencedata 
				  where dcdm_referencedata.reflistid=i_reflistid 
		                AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid))
	                    AND dcdm_referencedata.subid>=i_newsubid
                 ORDER BY dcdm_referencedata.subid desc;
 v_id int; 
BEGIN
	-- Сперва проверим соответствует ли тип справочкика заявленым параметрам
 IF EXISTS (SELECT * FROM dcdm_referenceslist where dcdm_referenceslist.id=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referenceslist.iswelldep=true) OR (i_wellid IS NULL AND dcdm_referenceslist.iswelldep=false))) THEN
  -- Узнаем текущую позицию элемента
  select dcdm_referencedata.subid into v_oldsubid from dcdm_referencedata
   where dcdm_referencedata.id=i_id AND dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid));
  IF v_oldsubid<>i_newsubid THEN
   -- проверим свободен ли указанный идентификатор
   IF EXISTS(select * FROM dcdm_referencedata where dcdm_referencedata.subid=i_newsubid AND dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid))) THEN
    -- Место занято и надо сдвигать subid
    OPEN curs;
    loop
	 FETCH FROM curs INTO v_id;
	 EXIT WHEN NOT FOUND;
	 UPDATE dcdm_referencedata SET subid=dcdm_referencedata.subid+1 WHERE dcdm_referencedata.id=v_id;
    end loop;
    close curs;
    UPDATE dcdm_referencedata SET subid=i_newsubid WHERE dcdm_referencedata.id=i_id AND dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid));
    GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
   ELSE
    -- место свободно
    UPDATE dcdm_referencedata SET subid=i_newsubid WHERE dcdm_referencedata.id=i_id AND dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid));
    GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
   END IF;
  ELSE
   ResulrCount:=0;
  END IF;
  RETURN ResulrCount;
 ELSE 
  RAISE EXCEPTION 'Invalid reference or well id' USING HINT='DCDM', DETAIL='RefIdOrWellIdErr';
 END IF;
END;
$$;
GRANT EXECUTE ON FUNCTION  ReposRefDataItem(dcdm_referencedata.id%TYPE,dcdm_referencedata.reflistid%TYPE,dcdm_referencedata.wellid%TYPE,dcdm_referencedata.subid%TYPE) TO dcdmsrv;
--#################################################################################################					 
/*
Очистка справочника
 $1 - Идентификатор справочника
 $2 - Идентификатор скважины для оперативных справочников или NULL для общих
*/
CREATE OR REPLACE FUNCTION ClearRefData(i_reflistid dcdm_referencedata.reflistid%TYPE,i_wellid dcdm_referencedata.wellid%TYPE)
RETURNS void 
VOLATILE LANGUAGE 'plpgsql' AS $$
#variable_conflict use_variable
BEGIN
-- Сперва проверим соответствует ли тип справочкика заявленым параметрам
 IF EXISTS (SELECT * FROM dcdm_referenceslist where dcdm_referenceslist.id=i_reflistid AND ((i_wellid IS NOT NULL AND dcdm_referenceslist.iswelldep=true) OR (i_wellid IS NULL AND dcdm_referenceslist.iswelldep=false))) THEN
  DELETE FROM dcdm_referencedata where dcdm_referencedata.reflistid=i_reflistid AND ((i_wellid IS NULL AND dcdm_referencedata.wellid IS NULL) OR (i_wellid IS NOT NULL AND dcdm_referencedata.wellid=i_wellid));  
 END IF;																													   
END;
$$;
GRANT EXECUTE ON FUNCTION  ClearRefData(dcdm_referencedata.reflistid%TYPE,dcdm_referencedata.wellid%TYPE) TO dcdmsrv;
--#################################################################################################					 
/*
Сохранение параметра конфигурации
 $1 - Ключ
 $2 - Значение
*/

CREATE OR REPLACE FUNCTION SaveConfigParam(i_key dcdm_config.key%TYPE,i_data dcdm_config.data%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResultCount int;
BEGIN
 INSERT INTO dcdm_config (key, data) VALUES (i_key,i_data) ON CONFLICT(key) DO UPDATE SET data=i_data;
 GET DIAGNOSTICS ResultCount:=ROW_COUNT;
 RETURN ResultCount;
END
$$;
GRANT EXECUTE ON FUNCTION  SaveConfigParam(dcdm_config.key%TYPE,dcdm_config.data%TYPE) TO dcdmsrv;
--#################################################################################################					 
/*
Сохранение конфигурации интерфейса пользователя
 $1 - идентификатор пользователя
 $2 - json объект интерфейса
*/
CREATE OR REPLACE FUNCTION SaveuserInterfaceCfg(i_id dcdm_users.id%TYPE,i_cfg dcdm_users.config%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResultCount int;
BEGIN
 UPDATE dcdm_users SET config=i_cfg WHERE id=i_id;  
 GET DIAGNOSTICS ResultCount:=ROW_COUNT;
 RETURN ResultCount;
END
$$;
GRANT EXECUTE ON FUNCTION  SaveuserInterfaceCfg(dcdm_users.id%TYPE,dcdm_users.config%TYPE) TO dcdmsrv;
--#################################################################################################					 
/*
Сохранение настроек пользователя
 $1 - идентификатор пользователя
 $2 - json объект настроек
*/
CREATE OR REPLACE FUNCTION SaveuserProps(i_id dcdm_users.id%TYPE,i_prop dcdm_users.props%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResultCount int;
BEGIN
 UPDATE dcdm_users SET props=i_prop WHERE id=i_id;  
 GET DIAGNOSTICS ResultCount:=ROW_COUNT;
 RETURN ResultCount;
END
$$;
GRANT EXECUTE ON FUNCTION  SaveuserProps(dcdm_users.id%TYPE,dcdm_users.props%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeAcGroupData(i_id dcdm_accessgroups.id%TYPE,i_name dcdm_accessgroups.name%TYPE)
RETURNS TABLE(id dcdm_accessgroups.id%TYPE,name dcdm_accessgroups.name%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_accessgroups(name) VALUES (i_name);  
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_accessgroups SET name=i_name WHERE dcdm_accessgroups.id=i_id;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY SELECT dcdm_accessgroups.id, dcdm_accessgroups.name
   FROM dcdm_accessgroups 
   WHERE dcdm_accessgroups.id=i_id;
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeAcGroupData(dcdm_accessgroups.id%TYPE,dcdm_accessgroups.name%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveAcGroupItem(i_id dcdm_accessgroups.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_accessgroups WHERE dcdm_accessgroups.id=i_id;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveAcGroupItem(dcdm_accessgroups.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION SaveAcGrpFldList(i_grpid dcdm_acgroupsfld.acgroupid%TYPE,i_fields varchar)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 -- сперва удалим старые поля
 DELETE FROM dcdm_acgroupsfld WHERE dcdm_acgroupsfld.acgroupid=i_grpid; 
 -- добавим новые
 INSERT INTO dcdm_acgroupsfld(acgroupid,name) SELECT i_grpid,json_array_elements_text(i_fields::json);
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  SaveAcGrpFldList(dcdm_acgroupsfld.acgroupid%TYPE,varchar) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION AddChangeCommentAndFile(
i_id      dcdm_commentsandfiles.id%TYPE,
i_wellid  dcdm_commentsandfiles.wellid%TYPE,
i_userid  dcdm_commentsandfiles.userid%TYPE,
i_type    dcdm_commentsandfiles.type%TYPE,
i_sdepth  dcdm_commentsandfiles.snapdepth%TYPE,
i_svdepth dcdm_commentsandfiles.snapvdepth%TYPE,
i_stime   dcdm_commentsandfiles.snaptime%TYPE,
i_comment dcdm_commentsandfiles.comment%TYPE,
i_fname   dcdm_commentsandfiles.orgfilename%TYPE,
i_fdata   dcdm_commentsandfiles.filedata%TYPE
)
RETURNS TABLE(
id dcdm_commentsandfiles.id%TYPE,
wid dcdm_commentsandfiles.wellid%TYPE,
uid	dcdm_commentsandfiles.userid%TYPE,
un  dcdm_users.name%TYPE,
crd float8,
chd float8,
t   dcdm_commentsandfiles.type%TYPE,
tn  dcdm_commentsandfilestype.name%TYPE,
isi dcdm_commentsandfiles.isindexed%TYPE,
sd  dcdm_commentsandfiles.snapdepth%TYPE,
svd dcdm_commentsandfiles.snapvdepth%TYPE,
st  float8,
msg dcdm_commentsandfiles.comment%TYPE,
fn  dcdm_commentsandfiles.orgfilename%TYPE,
fs  int
) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  -- Добавление
  INSERT INTO dcdm_commentsandfiles(wellid,userid,type,snapdepth,snapvdepth,snaptime,comment,orgfilename,filedata) 
  VALUES (i_wellid,i_userid,i_type,i_sdepth,i_svdepth,i_stime,i_comment,i_fname,i_fdata);
  select lastval() into i_id;
 else 
  -- Редактирование 
  UPDATE dcdm_commentsandfiles SET changetime=CURRENT_TIMESTAMP, type=i_type, isindexed=false,
                        snapdepth=i_sdepth,snapvdepth=i_svdepth,snaptime=i_stime,
						comment=i_comment--,orgfilename=i_fname,filedata=i_fdata
  WHERE dcdm_commentsandfiles.id=i_id AND dcdm_commentsandfiles.wellid=i_wellid AND dcdm_commentsandfiles.userid=i_userid;
 end if;
 -- Вернём отредактированный результат
 RETURN QUERY select dcdm_commentsandfiles.id,dcdm_commentsandfiles.wellid as wid,dcdm_commentsandfiles.userid as uid,dcdm_users.name as un,
                     date_part('epoch',dcdm_commentsandfiles.createtime) as crd,date_part('epoch',dcdm_commentsandfiles.changetime) as chd,dcdm_commentsandfiles.type as t,dcdm_commentsandfilestype.name as tn,
					 dcdm_commentsandfiles.isindexed as isi,dcdm_commentsandfiles.snapdepth as sd,dcdm_commentsandfiles.snapvdepth as svd,date_part('epoch',dcdm_commentsandfiles.snaptime) as st,
					 dcdm_commentsandfiles.comment as msg,dcdm_commentsandfiles.orgfilename as fn,octet_length(dcdm_commentsandfiles.filedata)
  from dcdm_commentsandfiles 
       inner join dcdm_users on dcdm_users.id=dcdm_commentsandfiles.userid
	   inner join dcdm_commentsandfilestype on dcdm_commentsandfilestype.id=dcdm_commentsandfiles.type
  where dcdm_commentsandfiles.id=i_id; 
END
$$;
GRANT EXECUTE ON FUNCTION  AddChangeCommentAndFile(dcdm_commentsandfiles.id%TYPE,dcdm_commentsandfiles.wellid%TYPE,dcdm_commentsandfiles.userid%TYPE,
							                        dcdm_commentsandfiles.type%TYPE,dcdm_commentsandfiles.snapdepth%TYPE,
												    dcdm_commentsandfiles.snapvdepth%TYPE,dcdm_commentsandfiles.snaptime%TYPE,dcdm_commentsandfiles.comment%TYPE,
												    dcdm_commentsandfiles.orgfilename%TYPE,dcdm_commentsandfiles.filedata%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION GetFilteredCommentList(
i_start bigint,	
i_wellid dcdm_commentsandfiles.wellid%TYPE,
i_orderby varchar(5),
i_isc bool,
i_isf bool,
i_tytes int[],
i_rt varchar(3),	
i_dfrom float8,
i_dto float8,	
i_tfrom timestamp,
i_tto timestamp	
)
RETURNS TABLE(
id dcdm_commentsandfiles.id%TYPE,
wid dcdm_commentsandfiles.wellid%TYPE,
uid	dcdm_commentsandfiles.userid%TYPE,
un  dcdm_users.name%TYPE,
crd float8,
chd float8,
t   dcdm_commentsandfiles.type%TYPE,
tn  dcdm_commentsandfilestype.name%TYPE,
isi dcdm_commentsandfiles.isindexed%TYPE,
sd  dcdm_commentsandfiles.snapdepth%TYPE,
svd dcdm_commentsandfiles.snapvdepth%TYPE,
st  float8,
msg dcdm_commentsandfiles.comment%TYPE,
fn  dcdm_commentsandfiles.orgfilename%TYPE,
fs  int
) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
DECLARE 
-- v_ordercolnum int;
 v_recperquery int:=100;
BEGIN
 -- Первый запрос вернет число записей
 IF i_start is NOT NULL then
  RETURN QUERY select i_start,count(*)::int,v_recperquery,null::varchar,null::float8,null::float8,
         null::smallint,null::varchar,null::bool,null::float8,null::float8,null::float8,null::varchar,
 		null::varchar,null::int
   from dcdm_commentsandfiles 
   where dcdm_commentsandfiles.wellid=i_wellid 
         AND dcdm_commentsandfiles.type=ANY(i_tytes) 
		 AND ((i_isc=true AND dcdm_commentsandfiles.comment IS NOT NULL) OR i_isc=false)
		 AND ((i_isf=true AND dcdm_commentsandfiles.filedata IS NOT NULL) OR i_isf=false)
		 AND ((i_rt='A')
 		     OR (i_rt='D' AND dcdm_commentsandfiles.snapdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='V' AND dcdm_commentsandfiles.snapvdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='T' AND dcdm_commentsandfiles.snaptime BETWEEN i_tfrom AND i_tto)
		    );
 end if;			
 -- Второй непосредственно результат			
 RETURN QUERY select dcdm_commentsandfiles.id,dcdm_commentsandfiles.wellid as wid,dcdm_commentsandfiles.userid as uid,dcdm_users.name as un,
                     date_part('epoch',dcdm_commentsandfiles.createtime) as crd,
					 date_part('epoch',dcdm_commentsandfiles.changetime) as chd,
					 dcdm_commentsandfiles.type as t,dcdm_commentsandfilestype.name as tn,
					 dcdm_commentsandfiles.isindexed as isi,
					 dcdm_commentsandfiles.snapdepth as sd,
					 dcdm_commentsandfiles.snapvdepth as svd,
					 date_part('epoch',dcdm_commentsandfiles.snaptime) as st,
					 dcdm_commentsandfiles.comment as msg,dcdm_commentsandfiles.orgfilename as fn,octet_length(dcdm_commentsandfiles.filedata)
  from dcdm_commentsandfiles 
       inner join dcdm_users on dcdm_users.id=dcdm_commentsandfiles.userid
	   inner join dcdm_commentsandfilestype on dcdm_commentsandfilestype.id=dcdm_commentsandfiles.type
  where dcdm_commentsandfiles.wellid=i_wellid 
        AND dcdm_commentsandfiles.type=ANY(i_tytes)
		AND ((i_isc=true AND dcdm_commentsandfiles.comment IS NOT NULL) OR i_isc=false)
		AND ((i_isf=true AND dcdm_commentsandfiles.filedata IS NOT NULL) OR i_isf=false)
		AND ((i_rt='A')
		     OR (i_rt='D' AND dcdm_commentsandfiles.snapdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='V' AND dcdm_commentsandfiles.snapvdepth BETWEEN i_dfrom AND i_dto)
		     OR (i_rt='T' AND dcdm_commentsandfiles.snaptime BETWEEN i_tfrom AND i_tto)
		    )
  order by 
   case when i_orderby='CrT' then dcdm_commentsandfiles.createtime end,
   case when i_orderby='ChT' then dcdm_commentsandfiles.changetime end,
   case when i_orderby='Dp' then dcdm_commentsandfiles.snapdepth end,
   case when i_orderby='VDp' then dcdm_commentsandfiles.snapvdepth end,
   case when i_orderby='EvT' then  dcdm_commentsandfiles.snaptime end
  limit case when i_start is not null then v_recperquery else null end offset case when i_start is not null then i_start else 0 end; 
END
$$;
GRANT EXECUTE ON FUNCTION  GetFilteredCommentList(bigint,dcdm_commentsandfiles.wellid%TYPE,
	   varchar(5),bool,bool,int[],varchar(3),float8,float8,timestamp,timestamp) TO dcdmsrv;

--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveCommentItem(i_WellId dcdm_commentsandfiles.wellid%TYPE,i_CommentId dcdm_commentsandfiles.id%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_commentsandfiles WHERE dcdm_commentsandfiles.id=i_CommentId AND dcdm_commentsandfiles.wellid=i_WellId;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveCommentItem(dcdm_commentsandfiles.wellid%TYPE,dcdm_commentsandfiles.id%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION BuildGetRawDataQuery(
i_start bigint,	
i_limit int,	
i_wellid dcdm_welllog.wellid%TYPE,
i_dfrom dcdm_welllog.depth%TYPE,
i_dto dcdm_welllog.depth%TYPE,	
i_vdfrom dcdm_welllog.vdepth%TYPE,
i_vdto dcdm_welllog.vdepth%TYPE,	
i_tfrom dcdm_welllog.time%TYPE,
i_tto dcdm_welllog.time%TYPE,
i_params text	
)
RETURNS varchar
IMMUTABLE LANGUAGE 'plpgsql' AS $body$
DECLARE 
	v_firstquery varchar:='SELECT '||
		case when i_vdfrom IS NOT NULL AND i_vdto IS NOT NULL then 'vdepth' when i_tfrom IS NOT NULL AND i_tto IS NOT NULL then 'time' else 'depth'	end
		||', paramname, val as val FROM dcdm_welllog where wellid='||i_wellid||' and (isnew is null or isnew=false) AND paramname=any(string_to_array('||quote_literal(i_params)||','',''))'||
		case when i_dfrom IS NOT NULL AND i_dto IS NOT NULL then ' and depth between '||i_dfrom||' and '||i_dto
			 when i_vdfrom IS NOT NULL AND i_vdto IS NOT NULL then ' and vdepth between '||i_vdfrom||' and '||i_vdto
			 when i_tfrom IS NOT NULL AND i_tto IS NOT NULL then ' and time between '||quote_literal(i_tfrom)||' and '||quote_literal(i_tto) else '' 
		end||' order by 1';
	v_secondquery varchar:='SELECT unnest(string_to_array('||quote_literal(i_params)||','',''))';
	v_t varchar;
	v_ctparam varchar:='';
	v_basequery varchar;
BEGIN
 FOREACH  v_t in array string_to_array(i_params,',') LOOP v_ctparam:=v_ctparam||', "'||v_t||'" json'; END LOOP;	
 v_basequery:='select * from crosstab('||quote_literal(v_firstquery)||','||quote_literal(v_secondquery)||') as ct('||
 	case when i_vdfrom IS NOT NULL AND i_vdto IS NOT NULL then 'vdepth float8' when i_tfrom IS NOT NULL AND i_tto IS NOT NULL then 'time timestamp' else 'depth float8'	end||
	v_ctparam||')'|| case when i_limit is not null then ' limit '||i_limit||' offset '||i_start||';' else ';' end; 
 return v_basequery;
END
$body$;
GRANT EXECUTE ON FUNCTION  BuildGetRawDataQuery(bigint,int,dcdm_commentsandfiles.wellid%TYPE,
 dcdm_welllog.depth%TYPE,dcdm_welllog.depth%TYPE,	
 dcdm_welllog.vdepth%TYPE,dcdm_welllog.vdepth%TYPE,	
 dcdm_welllog.time%TYPE,dcdm_welllog.time%TYPE,text) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION GetCommentsListByIds(
i_start bigint,	
i_idslist varchar
)
RETURNS TABLE(
id dcdm_commentsandfiles.id%TYPE,
wid dcdm_commentsandfiles.wellid%TYPE,
uid	dcdm_commentsandfiles.userid%TYPE,
un  dcdm_users.name%TYPE,
crd float8,
chd float8,
t   dcdm_commentsandfiles.type%TYPE,
tn  dcdm_commentsandfilestype.name%TYPE,
isi dcdm_commentsandfiles.isindexed%TYPE,
sd  dcdm_commentsandfiles.snapdepth%TYPE,
svd dcdm_commentsandfiles.snapvdepth%TYPE,
st  float8,
msg dcdm_commentsandfiles.comment%TYPE,
fn  dcdm_commentsandfiles.orgfilename%TYPE,
fs  int,
pr	int
) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
DECLARE 
 v_recperquery int:=100;
BEGIN
 -- Первый запрос вернет число записей
 RETURN QUERY select i_start,count(*)::int,v_recperquery,null::varchar,null::float8,null::float8,
        null::smallint,null::varchar,null::bool,null::float8,null::float8,null::float8,null::varchar,
		null::varchar,null::int,null::int
  FROM unnest(i_idslist::varchar[]) list_id LEFT JOIN dcdm_commentsandfiles t on t.id=substring(list_id from '^(\d+)')::bigint;		
 -- Второй непосредственно результат			
 RETURN QUERY select dcdm_commentsandfiles.id,dcdm_commentsandfiles.wellid as wid,dcdm_commentsandfiles.userid as uid,dcdm_users.name as un,
                     date_part('epoch',dcdm_commentsandfiles.createtime) as crd,date_part('epoch',dcdm_commentsandfiles.changetime) as chd,dcdm_commentsandfiles.type as t,dcdm_commentsandfilestype.name as tn,
					 dcdm_commentsandfiles.isindexed as isi,dcdm_commentsandfiles.snapdepth as sd,dcdm_commentsandfiles.snapvdepth as svd,date_part('epoch',dcdm_commentsandfiles.snaptime) as st,
					 dcdm_commentsandfiles.comment as msg,dcdm_commentsandfiles.orgfilename as fn,octet_length(dcdm_commentsandfiles.filedata),
			         substring(list_id from ':(\d+)')::int as pr
  FROM unnest(i_idslist::varchar[]) list_id LEFT JOIN dcdm_commentsandfiles on dcdm_commentsandfiles.id=substring(list_id from '^(\d+)')::bigint		
       inner join dcdm_users on dcdm_users.id=dcdm_commentsandfiles.userid
	   inner join dcdm_commentsandfilestype on dcdm_commentsandfilestype.id=dcdm_commentsandfiles.type
 order by pr desc,dcdm_commentsandfiles.id desc	
  limit v_recperquery offset i_start; 
END
$$;
GRANT EXECUTE ON FUNCTION  GetCommentsListByIds(bigint,varchar) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION GetPreliminaryWellLogData(
i_wellid dcdm_welllog.wellid%TYPE,
i_depth dcdm_welllog.depth%TYPE,
i_vdepth dcdm_welllog.vdepth%TYPE,
i_time dcdm_welllog.time%TYPE,
i_params text	
)
RETURNS TABLE(paramname dcdm_welllog.paramname%TYPE,val dcdm_welllog.val%TYPE,type smallint) 
IMMUTABLE LANGUAGE 'plpgsql' AS $$
BEGIN
 IF i_depth IS NOT NULL then 
   RETURN QUERY select distinct on (dcdm_welllog.paramname) dcdm_welllog.paramname, dcdm_welllog.val,1::smallint as type
   FROM dcdm_welllog 
   where dcdm_welllog.wellid=i_wellid AND (dcdm_welllog.isnew is null or dcdm_welllog.isnew=false) AND dcdm_welllog.paramname=any(string_to_array(i_params,',')) and dcdm_welllog.depth <i_depth
   order by dcdm_welllog.paramname, dcdm_welllog.depth desc;	
   RETURN QUERY select dcdm_welllog.paramname,('{"val":'||max(dcdm_welllog.depth)||'}')::jsonb,2::smallint
   FROM dcdm_welllog 
   where dcdm_welllog.wellid=i_wellid AND (dcdm_welllog.isnew is null or dcdm_welllog.isnew=false) AND dcdm_welllog.paramname=any(string_to_array(i_params,','))
   group by dcdm_welllog.paramname;																					 
 ELSEIF i_vdepth IS NOT NULL then 
   RETURN QUERY select distinct on (dcdm_welllog.paramname) dcdm_welllog.paramname, dcdm_welllog.val,1::smallint as type
   FROM dcdm_welllog 
   where dcdm_welllog.wellid=i_wellid AND (dcdm_welllog.isnew is null or dcdm_welllog.isnew=false) AND dcdm_welllog.paramname=any(string_to_array(i_params,',')) and dcdm_welllog.vdepth <i_vdepth
   order by dcdm_welllog.paramname, dcdm_welllog.vdepth desc;
   RETURN QUERY select dcdm_welllog.paramname,('{"val":'||max(dcdm_welllog.vdepth)||'}')::jsonb,2::smallint
   FROM dcdm_welllog 
   where dcdm_welllog.wellid=i_wellid AND (dcdm_welllog.isnew is null or dcdm_welllog.isnew=false) AND dcdm_welllog.paramname=any(string_to_array(i_params,','))
   group by dcdm_welllog.paramname;																					 
 ELSEIF i_time IS NOT NULL then 
   RETURN QUERY select distinct on (dcdm_welllog.paramname) dcdm_welllog.paramname, dcdm_welllog.val,1::smallint as type
   FROM dcdm_welllog 
   where dcdm_welllog.wellid=i_wellid AND (dcdm_welllog.isnew is null or dcdm_welllog.isnew=false) AND dcdm_welllog.paramname=any(string_to_array(i_params,',')) and dcdm_welllog.time <i_time
   order by dcdm_welllog.paramname, dcdm_welllog.time desc;
   RETURN QUERY select dcdm_welllog.paramname,('{"val":'||trunc(date_part('epoch',max(dcdm_welllog.time))*1000)||'}')::jsonb,2::smallint
   FROM dcdm_welllog 
   where dcdm_welllog.wellid=i_wellid AND (dcdm_welllog.isnew is null or dcdm_welllog.isnew=false) AND dcdm_welllog.paramname=any(string_to_array(i_params,','))
   group by dcdm_welllog.paramname;																					 
 END IF;																					 
END
$$;
GRANT EXECUTE ON FUNCTION  GetPreliminaryWellLogData(dcdm_welllog.wellid%TYPE,dcdm_welllog.depth%TYPE,dcdm_welllog.vdepth%TYPE,dcdm_welllog.time%TYPE,text) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION commitunfixeddata(i_userid dcdm_welllog.userid%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 -- сперва удалим старые записи
 DELETE FROM dcdm_welllog WHERE isold=true and (i_userid is null or (i_userid is not null and userid=i_userid)); 
 -- Зафиксируем новые
 UPDATE dcdm_welllog SET userid=null,isnew=null WHERE isnew=true and (i_userid is null or (i_userid is not null and userid=i_userid));  
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  commitunfixeddata(dcdm_welllog.userid%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION dropunfixeddata(i_userid dcdm_welllog.userid%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 -- сперва удалим новые записи
 DELETE FROM dcdm_welllog WHERE isnew=true and (i_userid is null or (i_userid is not null and userid=i_userid)); 
 -- вернем старые
 UPDATE dcdm_welllog SET userid=null,isold=null WHERE isold=true and (i_userid is null or (i_userid is not null and userid=i_userid));  
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  dropunfixeddata(dcdm_welllog.userid%TYPE) TO dcdmsrv;
--#################################################################################################					 
SET plpgsql.extra_warnings TO 'shadowed_variables';
CREATE OR REPLACE FUNCTION AddChangeLagData(
 i_oldname dcdm_lags.sysname%TYPE,
 i_name dcdm_lags.sysname%TYPE,
 i_init dcdm_lags.initvar%TYPE,
 i_trunc dcdm_lags.truncvar%TYPE)
RETURNS TABLE(
 sysname dcdm_lags.sysname%TYPE,
 initvar dcdm_lags.initvar%TYPE,
 /*idesc dcdm_vars.description%TYPE,*/	
 truncvar dcdm_lags.truncvar%TYPE
 /*,tdesc dcdm_vars.description%TYPE*/) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 IF i_oldname IS NULL then
  -- Добавление
  INSERT INTO dcdm_lags(sysname,initvar, truncvar) VALUES (i_name,i_init,i_trunc); 
 else
  -- редактирование
  UPDATE dcdm_lags SET sysname=i_name, initvar=i_init, truncvar=i_trunc WHERE dcdm_lags.sysname=i_oldname;  
 end if;
 
 RETURN QUERY SELECT dcdm_lags.sysname, dcdm_lags.initvar,/*vinit.description as idesc,*/ dcdm_lags.truncvar/*,vtrunc.description as tdesc*/
  FROM dcdm_lags
  /*inner join dcdm_vars vinit on vinit.sysname=dcdm_lags.initvar
  inner join dcdm_vars vtrunc on vtrunc.sysname=dcdm_lags.truncvar*/
  where dcdm_lags.sysname=i_name;

 EXCEPTION 
  WHEN unique_violation then RAISE EXCEPTION 'Name must be unique' USING HINT='DCDM', DETAIL='LagNameError';
  WHEN check_violation then RAISE EXCEPTION 'Invalid character' USING HINT='DCDM', DETAIL='LagInvalidChar';
END
$$;

GRANT EXECUTE ON FUNCTION  AddChangeLagData(dcdm_lags.sysname%TYPE,dcdm_lags.sysname%TYPE,dcdm_lags.initvar%TYPE,dcdm_lags.truncvar%TYPE)
TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION RemoveLagItem(i_sysname dcdm_lags.sysname%TYPE)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE
 ResulrCount int;
BEGIN
 DELETE FROM dcdm_lags WHERE dcdm_lags.sysname=i_sysname;
 GET DIAGNOSTICS ResulrCount:=ROW_COUNT;
 RETURN ResulrCount;
END
$$;
GRANT EXECUTE ON FUNCTION  RemoveLagItem(dcdm_lags.sysname%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION addwelllogrec(
i_acgrp dcdm_accessgroups.id%TYPE,
i_wellid dcdm_well.id%TYPE,
i_paramname dcdm_welllog.paramname%TYPE,
i_type varchar(10),
i_pos float8,
i_val jsonb) 
RETURNS TABLE(ids dcdm_welllog.id%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 IF EXISTS (select 1 from dcdm_acgroupsfld where acgroupid=i_acgrp and name=i_paramname) THEN
  if i_type='depth' then
   RETURN QUERY INSERT INTO dcdm_welllog(wellid,mode,"time",depth,vdepth,paramname,val) VALUES (i_wellid,0,null,i_pos,null,i_paramname,i_val) returning id;
  elsif  i_type='vdepth' then
   RETURN QUERY INSERT INTO dcdm_welllog(wellid,mode,"time",depth,vdepth,paramname,val) VALUES (i_wellid,0,null,null,i_pos,i_paramname,i_val) returning id;
  elsif i_type='time' then
   RETURN QUERY INSERT INTO dcdm_welllog(wellid,mode,"time",depth,vdepth,paramname,val) VALUES (i_wellid,0,to_timestamp(i_pos),null,null,i_paramname,i_val) returning id;
  else
   RAISE EXCEPTION 'Unknown scale type' USING HINT='DCDM', DETAIL='UnkScaleType';
  end if;
 ELSE
  RAISE EXCEPTION 'Access denied' USING HINT='DCDM', DETAIL='AddWellLogAcDen';
 END IF;
END
$$;
GRANT EXECUTE ON FUNCTION  addwelllogrec(dcdm_accessgroups.id%TYPE,dcdm_well.id%TYPE,dcdm_welllog.paramname%TYPE,varchar(10),float8,jsonb) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION addchangemarker(
i_userid dcdm_markers.userid%TYPE,
i_wellid dcdm_markers.wellid%TYPE,
i_param dcdm_markers.type%TYPE,
i_id dcdm_markers.id%TYPE,
i_type varchar(10),
i_from float8,
i_to float8,
i_code dcdm_markers.code%TYPE)
RETURNS TABLE(id dcdm_markers.id%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 if i_id=0 then 
  INSERT INTO dcdm_markers(wellid, userid, type, depth, vdepth, "time", code)
  VALUES (i_wellid,i_userid,i_param,
    case when 'depth'=i_type then floatrange(i_from,i_to,'[]') else null end,
    case when 'vdepth'=i_type then floatrange(i_from,i_to,'[]') else null end,		
    case when 'time'=i_type then tsrange(to_timestamp(i_from)::timestamp,to_timestamp(i_to)::timestamp,'[]') else null end,			  
    i_code);
  select lastval() into i_id;
 else 
  UPDATE dcdm_markers SET userid=i_userid, changetime=CURRENT_TIMESTAMP,
    depth=case when 'depth'=i_type then floatrange(i_from,i_to,'[]') else null end,
    vdepth=case when 'vdepth'=i_type then floatrange(i_from,i_to,'[]') else null end,		
    "time"=case when 'time'=i_type then tsrange(to_timestamp(i_from)::timestamp,to_timestamp(i_to)::timestamp,'[]') else null end,			  
    code=i_code 
  WHERE dcdm_markers.id=i_id AND dcdm_markers.wellid=i_wellid AND dcdm_markers.type=i_param;
 end if;
 RETURN QUERY select dcdm_markers.id from dcdm_markers where dcdm_markers.id=i_id;
END
$$;
GRANT EXECUTE ON FUNCTION  addchangemarker(dcdm_markers.userid%TYPE,dcdm_markers.wellid%TYPE,dcdm_markers.type%TYPE,dcdm_markers.id%TYPE,varchar(10),float8,float8,dcdm_markers.code%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION SetSortVarPos(i_sysname dcdm_vars.sysname%TYPE,i_pos dcdm_vars.sortpos%TYPE)
RETURNS TABLE(
 sysname dcdm_vars.sysname%TYPE,
 sortpos dcdm_vars.sortpos%TYPE) 
VOLATILE LANGUAGE 'plpgsql' AS $$
BEGIN
 -- Увеличип номер позиции сортировки всех переменных которые больше чем наш номер
 UPDATE dcdm_vars SET sortpos=dcdm_vars.sortpos+1 WHERE dcdm_vars.sortpos>=i_pos;
 -- Обновим позицию у целевой переменной
 UPDATE dcdm_vars SET sortpos=i_pos WHERE dcdm_vars.sysname=i_sysname;
 
 RETURN QUERY select dcdm_vars.sysname,dcdm_vars.sortpos from dcdm_vars where dcdm_vars.sysname=i_sysname;
END
$$;

GRANT EXECUTE ON FUNCTION  SetSortVarPos(dcdm_vars.sysname%TYPE,dcdm_vars.sortpos%TYPE) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION addlythology(
i_acgroup dcdm_accessgroups.id%TYPE,
i_from dcdm_welllog.depth%TYPE,
i_to dcdm_welllog.depth%TYPE,
i_wellid dcdm_welllog.wellid%TYPE,
i_param dcdm_welllog.paramname%TYPE,
i_code int)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE 
 curcode jsonb;
 RecCount int;
BEGIN
 -- Проверим группу доступа
 IF EXISTS (SELECT id FROM dcdm_acgroupsfld where name=i_param and acgroupid=i_acgroup) THEN
  -- Определим есть ли еще записи за нижней границей диапазона
  IF EXISTS (SELECT id FROM dcdm_welllog where wellid=i_wellid AND paramname=i_param AND depth>i_to AND isnew is null or isnew=false order by depth limit 1) THEN 
    -- Определим чем заканчивается сейчас наш диапазон
    SELECT val into curcode FROM dcdm_welllog where wellid=i_wellid AND paramname=i_param AND depth<=i_to AND isnew is null or isnew=false order by depth desc limit 1;
  ELSE
    curcode:=jsonb_build_object('val',null);
  END IF;
  -- Удалим все что попадает в наш диапазон
  DELETE FROM dcdm_welllog	WHERE  wellid=i_wellid AND paramname=i_param AND depth between i_from and i_to AND isnew is null or isnew=false;
  -- Добавим данные начала диапазона
  INSERT INTO dcdm_welllog(wellid,mode,"time",depth,paramname, val)	VALUES (i_wellid,0,null,i_from,i_param,jsonb_build_object('val',i_code));
  GET DIAGNOSTICS RecCount:=ROW_COUNT;
  -- Добавим данные окончания диапазона только если завершение не null
  INSERT INTO dcdm_welllog(wellid,mode,"time",depth,paramname, val)	VALUES (i_wellid,0,null,i_to,i_param,curcode);  
 ELSE																													   
  RAISE EXCEPTION 'Forbidden' USING HINT='DCDM', DETAIL='Forbidden';																													   
 END IF;																													    
 RETURN RecCount;
END
$$;
GRANT EXECUTE ON FUNCTION  addlythology(dcdm_accessgroups.id%TYPE,dcdm_welllog.depth%TYPE,dcdm_welllog.depth%TYPE,dcdm_welllog.wellid%TYPE,dcdm_welllog.paramname%TYPE,int) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION addsludge(
i_acgroup dcdm_accessgroups.id%TYPE,
i_from dcdm_welllog.depth%TYPE,
i_to dcdm_welllog.depth%TYPE,
i_wellid dcdm_welllog.wellid%TYPE,
i_params json)
RETURNS integer VOLATILE LANGUAGE 'plpgsql' AS $$
DECLARE 
 curcodes json;
 RecCount int;
 NewParamCount int; -- число уникальных добавляемых параметров
 GrpParamCount int; -- число уникальных добавляемых параметров в указанной группе доступа
BEGIN
 -- Получим число добавляемых полй и полей в группе доступа
 select count(distinct obj->>'p') into NewParamCount FROM json_array_elements(i_params) obj;
 select count(distinct obj->>'p') into GrpParamCount FROM json_array_elements(i_params) obj inner join dcdm_acgroupsfld a on a.name=obj->>'p';
 -- Проверим совпадают ли счетчики
 IF NewParamCount=GrpParamCount THEN
   -- Определим есть ли еще записи за нижней границей диапазона хотябы для одного из наших параметров
  IF EXISTS (SELECT id FROM dcdm_welllog where wellid=i_wellid AND paramname in (SELECT obj->>'p' FROM   json_array_elements(i_params) obj) AND depth>i_to AND isnew is null or isnew=false order by depth limit 1) THEN 
    -- Запомним чем заканчивается сейчас наш диапазон чтоб потом вставить эти данные после добавляемого блока
	SELECT json_agg(json_build_object('p',a.paramname,'v',a.val)) into curcodes FROM 
	(select distinct on(paramname) depth, w.paramname as paramname, w.val as val
	   FROM dcdm_welllog w inner join dcdm_acgroupsfld a on a.name=w.paramname
	  where a.acgroupid=i_acgroup
		and (w.isnew is null or w.isnew=false)
		and w.wellid=i_wellid
		and w.paramname in(SELECT obj->>'p' FROM json_array_elements(i_params) obj)
		and w.depth<=i_to
	  order by paramname,depth desc) a;	
  ELSE
    -- Ели данных дальше нет то создадим заглушку с null значениями для всез наших параметров
	SELECT json_agg(json_build_object('p',obj->>'p','v',json_build_object('val',null))) into curcodes FROM json_array_elements(i_params) obj;
  END IF;
  -- Удалим все что попадает в наш диапазон
  DELETE FROM dcdm_welllog	WHERE  wellid=i_wellid AND paramname in(SELECT obj->>'p' FROM json_array_elements(i_params) obj) AND depth between i_from and i_to AND isnew is null or isnew=false;
  -- Добавим данные начала диапазона
  INSERT INTO dcdm_welllog(wellid,mode,"time",depth,paramname, val)	SELECT i_wellid,0,null,i_from,obj->>'p',json_build_object('val',(obj->>'v')::int) FROM json_array_elements(i_params) obj;
  GET DIAGNOSTICS RecCount:=ROW_COUNT;
  -- Добавим данные окончания диапазона
  INSERT INTO dcdm_welllog(wellid,mode,"time",depth,paramname, val) SELECT i_wellid,0,null,i_to,obj->>'p',(obj->>'v')::jsonb FROM json_array_elements(curcodes) obj;
 ELSE RAISE EXCEPTION 'Forbidden' USING HINT='DCDM', DETAIL='Forbidden'; END IF;																													    
 RETURN RecCount;
END
$$;
GRANT EXECUTE ON FUNCTION  addsludge(dcdm_accessgroups.id%TYPE,dcdm_welllog.depth%TYPE,dcdm_welllog.depth%TYPE,dcdm_welllog.wellid%TYPE,json) TO dcdmsrv;
--#################################################################################################					 
CREATE OR REPLACE FUNCTION ApplayParamAcGrpFilter(i_acgrp DCDM_Users.AcGroup%TYPE,i_param varchar)
RETURNS varchar(20)[] IMMUTABLE LANGUAGE 'plpgsql'
AS $$
BEGIN
 return (select array( select name from unnest(string_to_array(i_param,',')) pn
 inner join dcdm_acgroupsfld a on a.name=pn.pn where a.acgroupid=i_acgrp));
END
$$;

GRANT EXECUTE ON FUNCTION  ApplayParamAcGrpFilter(DCDM_Users.AcGroup%TYPE,varchar) TO dcdmsrv;
--#################################################################################################					 
--#################################################################################################					 
--#################################################################################################					 
--#################################################################################################					 
--#################################################################################################					 
--#################################################################################################					 
GRANT USAGE, SELECT ON ALL SEQUENCES IN SCHEMA public TO dcdmsrv;